Package: minter
Title: Effect Sizes for Meta-Analysis of Interactions from Factorial
        Experiments
Version: 0.1.1
Authors@R: c(
    person("Facundo", "Decunta", email = "fdecunta@agro.uba.ar", role = c("aut", "cre"),
           comment = c(ORCID = "0009-0004-5774-1085")),
    person("Shinichi", "Nakagawa", email = "shinichi.nakagawa@ualberta.ca", role = c("ctb")),
    person("Daniel", "Noble", email = "daniel.noble@anu.edu.au", role = c("ctb"))
    )
Description: Compute effect sizes and their sampling variances from factorial experimental designs. The package supports calculation of simple effects, overall effects, and interaction effects for use in factorial meta-analyses. See Gurevitch et al. (2000) <doi:10.1086/303337>, Morris et al. (2007) <doi:10.1890/06-0442>, Lajeunesse (2011) <doi:10.1890/11-0423.1> and Macartney et al. (2022) <doi:10.1016/j.neubiorev.2022.104554>.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://fdecunta.github.io/minter/
BugReports: https://github.com/fdecunta/minter/issues
Imports: checkmate
Depends: R (>= 3.5)
LazyData: true
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-03 15:18:52 UTC; fd
Author: Facundo Decunta [aut, cre] (ORCID:
    <https://orcid.org/0009-0004-5774-1085>),
  Shinichi Nakagawa [ctb],
  Daniel Noble [ctb]
Maintainer: Facundo Decunta <fdecunta@agro.uba.ar>
Repository: CRAN
Date/Publication: 2026-05-03 15:50:02 UTC
Built: R 4.6.0; ; 2026-05-03 18:06:45 UTC; unix

# Generated by roxygen2: do not edit by hand

export(SMD_ind)
export(SMD_inter)
export(SMD_main)
export(lnCVR_ind)
export(lnCVR_inter)
export(lnCVR_main)
export(lnRR_ind)
export(lnRR_inter)
export(lnRR_main)
export(lnVR_ind)
export(lnVR_inter)
export(lnVR_main)
export(time_SMD)
export(time_lnCVR)
export(time_lnRR)
export(time_lnVR)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# minter <img src="man/figures/minter_logo.png" align="right" width="120"/>

<!-- badges: start -->

[![R-CMD-check](https://github.com/fdecunta/minter/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/fdecunta/minter/actions/workflows/R-CMD-check.yaml)
[![Codecov test
coverage](https://codecov.io/gh/fdecunta/minter/graph/badge.svg)](https://app.codecov.io/gh/fdecunta/minter)
[![CRAN
status](https://www.r-pkg.org/badges/version/minter)](https://CRAN.R-project.org/package=minter)
<!-- badges: end -->

Effect sizes for meta-analysis of interactions. A collection of
functions for computing effect sizes from factorial experiments.

## Installation

From CRAN:

``` r
install.packages("minter")
```

From GitHub repo:

``` r
devtools::install_github("fdecunta/minter")
```

## Example

Imagine you need to estimate the effect size of the interaction between
fertilization × drought on plant biomass:

``` r
library(minter)

# Dummy data from 8 studies examining fertilization × drought on plant biomass
studies <- data.frame(
  study_id = 1:8,
  # Control: No fertilization, well-watered
  Ctrl_mean = c(12.3, 15.7, 10.8, 14.2, 11.9, 13.5, 16.1, 12.7),
  Ctrl_sd   = c(2.1, 3.2, 1.8, 2.7, 2.3, 2.9, 3.5, 2.4),
  Ctrl_n    = c(20, 24, 18, 22, 19, 25, 23, 21),
  # Fertilization only
  Fert_mean = c(18.5, 21.3, 16.2, 19.8, 17.1, 20.4, 22.7, 18.9),
  Fert_sd   = c(3.1, 4.1, 2.7, 3.6, 3.2, 3.8, 4.2, 3.4),
  Fert_n    = c(22, 25, 20, 24, 21, 26, 25, 23),
  # Drought only  
  Drought_mean = c(8.7, 11.2, 7.9, 10.1, 8.3, 9.8, 11.7, 9.4),
  Drought_sd   = c(1.8, 2.4, 1.6, 2.1, 1.9, 2.3, 2.6, 2.0),
  Drought_n    = c(19, 23, 17, 21, 18, 24, 22, 20),
  # Both treatments
  Both_mean = c(14.2, 17.8, 12.9, 16.3, 13.7, 16.1, 18.4, 15.2),
  Both_sd   = c(2.9, 3.7, 2.5, 3.3, 2.8, 3.4, 3.9, 3.1),
  Both_n    = c(21, 26, 19, 23, 20, 27, 24, 22)
)

# Calculate interaction effect: Does fertilization work differently under drought?
interaction_results <- lnRR_inter(
  data = studies,
  Ctrl_mean = "Ctrl_mean", Ctrl_sd = "Ctrl_sd", Ctrl_n = "Ctrl_n",
  A_mean = "Fert_mean", A_sd = "Fert_sd", A_n = "Fert_n", 
  B_mean = "Drought_mean", B_sd = "Drought_sd", B_n = "Drought_n",
  AB_mean = "Both_mean", AB_sd = "Both_sd", AB_n = "Both_n"
)

head(interaction_results)
#>   study_id Ctrl_mean Ctrl_sd ... yi        vi
#> 1        1      12.3     2.1     0.081  0.0069
#> 2        2      15.7     3.2     0.158  0.0068
#> 3        3      10.8     1.8     0.084  0.0073

# Meta-analysis
library(metafor)
res <- rma(yi, vi, ..., data = interaction_results)
```

## Effect Size Measures

- **lnRR**: Log Response Ratio - proportional effects
- **SMD**: Standardized Mean Difference - standardized effects  
- **lnVR**: Log Variability Ratio - effects on variability
- **lnCVR**: Log Coefficient of Variation Ratio - effects on relative
  variability

## Effect Types

- **Individual**: Simple treatment vs. control, the classic effect size
- **Main**: Overall effect across levels of another factor
- **Interaction**: Does one factor’s effect depend on another?
- **Time**: Treatment × time interactions for repeated measures

## Acknowledgments

Thanks to Shinichi Nakagawa and Daniel Noble for generously sharing
their formulas for meta-analysis of interactions.

<!-- Improved compatibility of back to top link: See: https://github.com/othneildrew/Best-README-Template/pull/73 -->
<a id="readme-top"></a>
<!--
*** Thanks for checking out the Best-README-Template. If you have a suggestion
*** that would make this better, please fork the repo and create a pull request
*** or simply open an issue with the tag "enhancement".
*** Don't forget to give the project a star!
*** Thanks again! Now go create something AMAZING! :D
-->



<!-- PROJECT SHIELDS -->
<!--
*** I'm using markdown "reference style" links for readability.
*** Reference links are enclosed in brackets [ ] instead of parentheses ( ).
*** See the bottom of this document for the declaration of the reference variables
*** for contributors-url, forks-url, etc. This is an optional, concise syntax you may use.
*** https://www.markdownguide.org/basic-syntax/#reference-style-links
-->


<!-- PROJECT LOGO -->
<br />
<div align="center">
  <a href="https://github.com/PythonNoob999/Minter">
    <img src="images/nft.jpg" alt="Logo" width="80" height="80">
  </a>

  <h3 align="center">Minter</h3>

  <p align="center">
    A tool to mint NFTs
    <br />
    <a href="https://github.com/PythonNoob999/Minter"><strong>Explore the docs »</strong></a>
    <br />
    <br />
    <a href="https://github.com/PythonNoob999/Minter/issues/new?labels=bug&template=bug-report---.md">Report Bug</a>
    ·
    <a href="https://github.com/PythonNoob999/Minter/issues/new?labels=enhancement&template=feature-request---.md">Request Feature</a>
  </p>
</div>



<!-- TABLE OF CONTENTS -->
<details>
  <summary>Table of Contents</summary>
  <ol>
    <li>
      <a href="#about-the-project">About The Project</a>
      <ul>
        <li><a href="#built-with">Built With</a></li>
      </ul>
    </li>
    <li>
      <a href="#getting-started">Getting Started</a>
      <ul>
        <li><a href="#prerequisites">Prerequisites</a></li>
        <li><a href="#installation">Installation</a></li>
      </ul>
    </li>
    <li><a href="#usage">Usage</a></li>
    <li><a href="#roadmap">Roadmap</a></li>
    <li><a href="#contributing">Contributing</a></li>
    <li><a href="#license">License</a></li>
    <li><a href="#contact">Contact</a></li>
    <li><a href="#acknowledgments">Acknowledgments</a></li>
  </ol>
</details>



<!-- ABOUT THE PROJECT -->
## About The Project

Minter is a python tool/lib to let you mint NFTs (mint = minting nfts from a nft collection, not creating nfts)

<p align="right">(<a href="#readme-top">back to top</a>)</p>



### Built With

* [Web3.py](https://github.com/ethereum/web3.py)
* [kvsqlite](https://github.com/AYMENJD/Kvsqlite)

<p align="right">(<a href="#readme-top">back to top</a>)</p>



<!-- GETTING STARTED -->
## Getting Started

this is how you can get started using Minter

### Prerequisites

* python
* web3
* kvsqlite

### Installation

Here are the instalation steps

#### Method 1 from source
1. Clone the repo
```sh
git clone https://github.com/PythonNoob999/Minter.git
```
2. Build lib
```sh
pip install . -U
```

#### Method 2 from pip (recommended)
1. install via pip
```sh
pip install minter
```

<p align="right">(<a href="#readme-top">back to top</a>)</p>



<!-- USAGE EXAMPLES -->
## Usage

here are some using examples


### initilazing your Minter
```python
from Minter import Minter
import asyncio

async def main():
    # the RPC for your wanted evm chain
    BASE_RPC = "https://mainnet.base.org"
    # semaphore for rate limitng, e.g semaphore=30 means max requests can be handeld per time is 30
    minter = Minter(BASE_RPC, semaphore=30)
    await minter.connect()

asyncio.run(main())
```

### Creating Wallets
```python
from Minter import Minter, generate_wallets
import asyncio

async def main():
    minter = Minter(BASE_RPC, semaphore=30)
    await minter.connect()
    # generate 10 new wallets
    wallets = generate_wallets(10)
    # save them
    await minter.storage.insert_data(wallets=wallets)

asyncio.run(main())
```

#### importing Wallets
```python
from Minter import Minter, Wallet
import asyncio

minter = Minter(BASE_RPC, semaphore=30)
await minter.connect()
wallets = [
    Wallet(
        "0x..",
        "ce.."
    ),
    ...
]
await minter.storage.insert_data(wallets=wallets)
```

_For more examples, please refer to the [Documentation](https://github.com/PythonNoob999/Minter/tree/main/examples)_

<p align="right">(<a href="#readme-top">back to top</a>)</p>



<!-- ROADMAP -->
## Todo
- [X] Add price option to mint_bulk
- [X] Add custom ABIs
- [X] Add get_nft_balances Method
- [X] Add send_nfts Method
- [ ] Add Documentaion
- [ ] Add Custom Exceptions
- [ ] Add More Storage Options

See the [open issues](https://github.com/PythonNoob999/Minter/issues) for a full list of proposed features (and known issues).

<p align="right">(<a href="#readme-top">back to top</a>)</p>



<!-- CONTRIBUTING -->
## Contributing

Any contributions you make are **greatly appreciated**.

If you have a suggestion that would make this better, please fork the repo and create a pull request. You can also simply open an issue with the tag "enhancement".
Don't forget to give the project a star! Thanks again!

1. Fork the Project
2. Create your Feature Branch (`git checkout -b feature/AmazingFeature`)
3. Commit your Changes (`git commit -m 'Add some AmazingFeature'`)
4. Push to the Branch (`git push origin feature/AmazingFeature`)
5. Open a Pull Request


<!-- LICENSE -->
## License

Distributed under the MIT License. See `LICENSE.txt` for more information.

<p align="right">(<a href="#readme-top">back to top</a>)</p>



<!-- CONTACT -->
## Contact

SpicyPneguin - [telegram](https://t.me/kerolis55463)

<p align="right">(<a href="#readme-top">back to top</a>)</p>

#' Log Response Ratio: Interaction Between Treatment and Time
#'
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param t0_Ctrl_mean Sample mean from the control group at time 0
#' @param t0_Ctrl_sd Standard deviation from the control group at time 0
#' @param t1_Ctrl_mean Sample mean from the control group at time 1
#' @param t1_Ctrl_sd Standard deviation from the control group at time 1
#' @param Ctrl_n Sample size of the control group
#' @param Ctrl_cor Number or numeric vector. Correlation between the means of the control group at t0 and t1
#' @param t0_Exp_mean Sample mean from the experimental group at time 0
#' @param t0_Exp_sd Standard deviation from the experimental group at time 0
#' @param t1_Exp_mean Sample mean from the experimental group at time 1
#' @param t1_Exp_sd Standard deviation from the experimental group at time 1
#' @param Exp_n Sample size of the experimental group
#' @param Exp_cor Number or numeric vector. Correlation between the means of the experimental group at t0 and t1
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @references 
#'   Shinichi Nakagawa and Daniel Noble, personal communication.
#'
#' @examples
#' data <- data.frame(
#'   study_id = 1:2,
#'   pre_control_mean = c(8.4, 10.2),     # Control before restoration
#'   pre_control_sd = c(1.8, 2.1),
#'   post_control_mean = c(8.9, 10.7),    # Control after restoration period
#'   post_control_sd = c(1.9, 2.2),
#'   control_n = c(22, 18),
#'   pre_restoration_mean = c(8.6, 10.1), # Restoration sites before
#'   pre_restoration_sd = c(1.9, 2.0),
#'   post_restoration_mean = c(15.3, 17.8), # Restoration sites after
#'   post_restoration_sd = c(3.2, 3.7),
#'   restoration_n = c(20, 19)
#' )
#' 
#' result <- time_lnRR(
#'   data = data,
#'   t0_Ctrl_mean = "pre_control_mean", t0_Ctrl_sd = "pre_control_sd",
#'   t1_Ctrl_mean = "post_control_mean", t1_Ctrl_sd = "post_control_sd",
#'   Ctrl_n = "control_n", Ctrl_cor = 0.7,  # Correlation within control sites
#'   t0_Exp_mean = "pre_restoration_mean", t0_Exp_sd = "pre_restoration_sd",
#'   t1_Exp_mean = "post_restoration_mean", t1_Exp_sd = "post_restoration_sd", 
#'   Exp_n = "restoration_n", Exp_cor = 0.6   # Correlation within restoration sites
#' )
#' 
#' # Using different correlations for each study
#' result2 <- time_lnRR(
#'   data = data,
#'   t0_Ctrl_mean = "pre_control_mean", t0_Ctrl_sd = "pre_control_sd",
#'   t1_Ctrl_mean = "post_control_mean", t1_Ctrl_sd = "post_control_sd",
#'   Ctrl_n = "control_n", Ctrl_cor = c(0.6, 0.8),
#'   t0_Exp_mean = "pre_restoration_mean", t0_Exp_sd = "pre_restoration_sd",
#'   t1_Exp_mean = "post_restoration_mean", t1_Exp_sd = "post_restoration_sd", 
#'   Exp_n = "restoration_n", Exp_cor = c(0.5, 0.7)
#' )
#'
#' @export
time_lnRR <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  t0_Ctrl_mean,
  t0_Ctrl_sd,
  t1_Ctrl_mean,
  t1_Ctrl_sd,
  Ctrl_n,
  Ctrl_cor,
  t0_Exp_mean,
  t0_Exp_sd,
  t1_Exp_mean,
  t1_Exp_sd,
  Exp_n,
  Exp_cor
) {
  .assert_args(col_names, append, data)
  .assert_cor_value(Ctrl_cor, data)
  .assert_cor_value(Exp_cor, data)

  call_args <- as.list(match.call())[-1]

  time_lnrr_func <- ".time_interaction_lnRR"
  time_lnrr_args <-.get_columns(call_args[.time_lnRR_requirements], data)

  time_lnrr_args$Ctrl_cor <- Ctrl_cor
  time_lnrr_args$Exp_cor <- Exp_cor

  df <- .compute_and_format(
    data = data,
    effsize_func = time_lnrr_func,
    effsize_args = time_lnrr_args,
    col_names = col_names,
    append = append
  )

  return(df)
}


.time_lnRR_requirements <- c(
   "t0_Ctrl_mean",
   "t0_Ctrl_sd",
   "t1_Ctrl_mean",
   "t1_Ctrl_sd",
   "Ctrl_n",
   "t0_Exp_mean",
   "t0_Exp_sd",
   "t1_Exp_mean",
   "t1_Exp_sd",
   "Exp_n"
)


#' Log Response Ratio: Interaction Between Experimental Treatment and Time
#'
#' @param t0_Ctrl_mean Sample mean from the control group at time 0
#' @param t0_Ctrl_sd Standard deviation from the control group at time 0
#' @param t1_Ctrl_mean Sample mean from the control group at time 1
#' @param t1_Ctrl_sd Standard deviation from the control group at time 1
#' @param Ctrl_n Sample size of the control group
#' @param Ctrl_cor Correlation between the means of the control group at t0 and t1
#' @param t0_Exp_mean Sample mean from the experimental group at time 0
#' @param t0_Exp_sd Standard deviation from the experimental group at time 0
#' @param t1_Exp_mean Sample mean from the experimental group at time 1
#' @param t1_Exp_sd Standard deviation from the experimental group at time 1
#' @param Exp_n Sample size of the experimental group
#' @param Exp_cor Correlation between the means of the experimental group at t0 and t1
#'
#' @keywords internal
.time_interaction_lnRR <- function(
   t0_Ctrl_mean,
   t0_Ctrl_sd,
   t1_Ctrl_mean,
   t1_Ctrl_sd,
   Ctrl_n,
   Ctrl_cor,
   t0_Exp_mean,
   t0_Exp_sd,
   t1_Exp_mean,
   t1_Exp_sd,
   Exp_n,
   Exp_cor
) {
  lnRR <- log((t1_Exp_mean / t1_Ctrl_mean) / (t0_Exp_mean / t0_Ctrl_mean))

  lnRRv <- (
    (
      ((t0_Exp_mean^2 * t1_Exp_sd^2 + t1_Exp_mean^2 * t0_Exp_sd^2) -
      (2 * Exp_cor * t0_Exp_mean * t1_Exp_mean * t0_Exp_sd * t1_Exp_sd)) /
      (Exp_n * t0_Exp_mean^2 * t1_Exp_mean^2)
    ) +
    (
      ((t0_Ctrl_mean^2 * t1_Ctrl_sd^2 + t1_Ctrl_mean^2 * t0_Ctrl_sd^2) -
      (2 * Ctrl_cor * t0_Ctrl_mean * t1_Ctrl_mean * t0_Ctrl_sd * t1_Ctrl_sd)) /
      (Ctrl_n * t0_Ctrl_mean^2 * t1_Ctrl_mean^2)
    )
  )

  return(data.frame(lnRR, lnRRv))
}

test_that(".time_interaction_lnRR works fine computing the interaction and its sampling variance", {
  res <- .time_interaction_lnRR(
    t0_Ctrl_mean = 10,
    t0_Ctrl_sd = 2,
    t1_Ctrl_mean = 5,
    t1_Ctrl_sd = 1,
    Ctrl_n = 10,
    Ctrl_cor = 0.5,
    t0_Exp_mean = 10,
    t0_Exp_sd = 2,
    t1_Exp_mean = 9,
    t1_Exp_sd = 2,
    Exp_n = 9,
    Exp_cor = 0.5
  )

  # Computed manually
  test_lnRR <- 0.5877866
  test_lnRRv <- 0.008993141

  expect_equal(res$lnRR, test_lnRR, tolerance = 1e-6)
  expect_equal(res$lnRRv, test_lnRRv, tolerance = 1e-6)
})


test_that("time_lnRR works fine", {
  df <- data.frame(
    t0_Cm = 10,
    t0_Csd = 2,
    t1_Cm = 5,
    t1_Csd = 1,
    Cn = 10,
    t0_Em = 10,
    t0_Esd = 2,
    t1_Em = 9,
    t1_Esd = 2,
    En = 9
  )

  res <- time_lnRR(
    data = df,
    t0_Ctrl_mean = t0_Cm,
    t0_Ctrl_sd = t0_Csd,
    t1_Ctrl_mean = t1_Cm,
    t1_Ctrl_sd = t1_Csd,
    Ctrl_n = Cn,
    Ctrl_cor = 0.5,
    t0_Exp_mean = t0_Em,
    t0_Exp_sd = t0_Esd,
    t1_Exp_mean = t1_Em,
    t1_Exp_sd = t1_Esd,
    Exp_n = En,
    Exp_cor = 0.5
  )

  # Computed manually
  test_lnRR <- 0.5877866
  test_lnRRv <- 0.008993141

  expect_equal(res$yi, test_lnRR, tolerance = 1e-6)
  expect_equal(res$vi, test_lnRRv, tolerance = 1e-6)

})

#' Simple effect: Log Response Ratio
#'
#' Computes the individual or simple effect of Factor A over the Control. 
#' 
#' It is the classic Log Response Ratio (lnRR), which can also be computed
#' with metafor's `escalc()` function using `measure = "ROM"`.
#'
#' See the package vignette for a detailed description of the formula.
#' 
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param Ctrl_mean Mean outcome from the Control treatment
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_mean Mean outcome from the experimental treatment
#' @param A_sd Standard deviation from the experimental treatment
#' @param A_n Sample size from the experimental treatment
#'
#' @return A data frame containing the effect sizes and their sampling variance.
#'   By default, the columns are named `yi` (effect size) and `vi` (sampling variance). 
#'   If `append = TRUE`, the results are appended to the input `data`; otherwise, only the computed effect size columns are returned.
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @references 
#'   Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
#'     Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
#'     effects of enemies and mutualists on plant performance: a meta‐analysis. 
#'     Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
#'
#'   Lajeunesse, M. J. (2011). On the meta‐analysis of response ratios for
#'     studies with correlated and multi‐group designs. Ecology, 92(11), 2049-2055.
#'     https://doi.org/10.1890/11-0423.1
#'
#' @examples
#' data <- data.frame(
#'   study_id = 1:3,
#'   control_mean = c(10, 15, 12),
#'   control_sd = c(2.1, 3.2, 2.8),
#'   control_n = c(20, 25, 18),
#'   drought_mean = c(12, 18, 14),
#'   drought_sd = c(2.3, 3.5, 3.1),
#'   drought_n = c(22, 24, 20)
#' )
#' 
#' # Compute individual effect of drought vs control
#' result <- lnRR_ind(
#'   data = data,
#'   Ctrl_mean = "control_mean",
#'   Ctrl_sd = "control_sd", 
#'   Ctrl_n = "control_n",
#'   A_mean = "drought_mean",
#'   A_sd = "drought_sd",
#'   A_n = "drought_n"
#' )
#'
#' @export
lnRR_ind <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n
) {
  .assert_args(col_names, append, data)

  call_args <- as.list(match.call())[-1]

  lnrr_args <- .get_columns(call_args[.lnRR_args$ind], data)

  df <- .compute_and_format(
    effsize_func = ".simple_lnRR",
    effsize_args = lnrr_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' Main effect: Log Response Ratio
#' 
#' Computes the main effect of Factor A across levels of Factor B, analogous
#' to the main effect in a factorial ANOVA. 
#'
#' See the package vignette for a detailed description of the formula.
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param method Method to compute lnRR. Can be either "nakagawa" or "morris". Default is "nakagawa".
#' @param Ctrl_mean Mean outcome from the Control treatment
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_mean Mean outcome from the A treatment
#' @param A_sd Standard deviation from the A treatment
#' @param A_n Sample size from the A treatment
#' @param B_mean Mean outcome from the B treatment
#' @param B_sd Standard deviation from the B treatment 
#' @param B_n Sample size from the B treatment 
#' @param AB_mean Mean outcome from the interaction AxB treatment
#' @param AB_sd Standard deviation from the interaction AxB treatment
#' @param AB_n Sample size from the interaction AxB treatment
#'
#' @inherit lnRR_ind return
#' 
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @references 
#'   Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
#'     Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
#'     effects of enemies and mutualists on plant performance: a meta‐analysis. 
#'     Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
#'
#'   Lajeunesse, M. J. (2011). On the meta‐analysis of response ratios for
#'     studies with correlated and multi‐group designs. Ecology, 92(11), 2049-2055.
#'     https://doi.org/10.1890/11-0423.1
#'
#'   Macartney, E. L., Lagisz, M., & Nakagawa, S. (2022). The relative benefits
#'     of environmental enrichment on learning and memory are greater when 
#'     stressed: A meta-analysis of interactions in rodents.
#'     Neuroscience & Biobehavioral Reviews, 135, 104554.
#'     https://doi.org/10.1016/j.neubiorev.2022.104554 
#'
#' @examples
#' # Example data for 2x2 factorial design (Fertilization x Warming)
#' data <- data.frame(
#'   study_id = 1:2,
#'   control_mean = c(10, 12), control_sd = c(2.0, 2.5), control_n = c(20, 18),
#'   fertilization_mean = c(15, 16), fertilization_sd = c(2.2, 2.8), fertilization_n = c(20, 19),
#'   warming_mean = c(11, 13), warming_sd = c(2.1, 2.6), warming_n = c(21, 17),
#'   fert_warm_mean = c(17, 19), fert_warm_sd = c(2.4, 3.0), fert_warm_n = c(19, 20)
#' )
#' 
#' # Compute main effect of fertilization
#' result <- lnRR_main(
#'   data = data,
#'   Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
#'   A_mean = "fertilization_mean", A_sd = "fertilization_sd", A_n = "fertilization_n",
#'   B_mean = "warming_mean", B_sd = "warming_sd", B_n = "warming_n",
#'   AB_mean = "fert_warm_mean", AB_sd = "fert_warm_sd", AB_n = "fert_warm_n"
#' )
#'
#' @export
lnRR_main <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  method = "nakagawa", 
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  .assert_args(col_names, append, data)
  checkmate::assert_choice(method, choices = c("nakagawa", "morris"))

  call_args <- as.list(match.call())[-1]

  lnrr_args <- .get_columns(call_args[.lnRR_args$main], data)

  func_name <- if (method == "nakagawa") ".main_lnRR_Nakagawa" else ".main_lnRR_Morris"

  df <- .compute_and_format(
    effsize_func = func_name,
    effsize_args = lnrr_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' Interaction effect: Log Response Ratio
#' 
#' Computes the interaction effect between factors A and B in factorial
#' data.
#'
#' See the package vignette for a detailed description of the formula.
#' 
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param Ctrl_mean Mean outcome from the Control treatment
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_mean Mean outcome from the treatment
#' @param A_sd Standard deviation from the treatment
#' @param A_n Sample size from the treatment
#' @param B_mean Mean outcome from the B treatment
#' @param B_sd Standard deviation from the B treatment 
#' @param B_n Sample size from the B treatment 
#' @param AB_mean Mean outcome from the interaction AxB treatment
#' @param AB_sd Standard deviation from the interaction AxB treatment
#' @param AB_n Sample size from the interaction AxB treatment
#'
#' @inherit lnRR_ind return
#' 
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @references 
#'   Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
#'     Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
#'     effects of enemies and mutualists on plant performance: a meta‐analysis. 
#'     Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
#'
#' @examples
#' data <- data.frame(
#'   study_id = 1:2,
#'   control_mean = c(25, 28), control_sd = c(3.2, 3.8), control_n = c(15, 17),
#'   predation_mean = c(18, 20), predation_sd = c(2.9, 3.1), predation_n = c(16, 18),
#'   competition_mean = c(22, 24), competition_sd = c(3.0, 3.5), competition_n = c(14, 16),
#'   pred_comp_mean = c(12, 15), pred_comp_sd = c(2.1, 2.6), pred_comp_n = c(15, 17)
#' )
#' 
#' # Compute interaction effect between predation and competition
#' result <- lnRR_inter(
#'   data = data,
#'   Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
#'   A_mean = "predation_mean", A_sd = "predation_sd", A_n = "predation_n",
#'   B_mean = "competition_mean", B_sd = "competition_sd", B_n = "competition_n",
#'   AB_mean = "pred_comp_mean", AB_sd = "pred_comp_sd", AB_n = "pred_comp_n"
#' )
#'
#' @export
lnRR_inter <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  .assert_args(col_names, append, data)

  call_args <- as.list(match.call())[-1]

  lnrr_args <- .get_columns(call_args[.lnRR_args$main], data)  # Same args as main

  df <- .compute_and_format(
    effsize_func = ".interaction_lnRR",
    effsize_args = lnrr_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' @keywords internal
.lnRR_args <- list(
  ind = c(
    "Ctrl_mean",
    "Ctrl_sd",  
    "Ctrl_n",   
    "A_mean",   
    "A_sd",     
    "A_n"
  ),
  main = c(
    "Ctrl_mean",
    "Ctrl_sd",  
    "Ctrl_n",   
    "A_mean",   
    "A_sd",     
    "A_n",
    "B_mean",
    "B_sd",
    "B_n",
    "AB_mean",
    "AB_sd",
    "AB_n"
  )
)

test_that("lnRR function works fine with 'ind' effect size", {
  res <- lnRR_ind(
    data = testing_data,
    col_names = c("simple_lnRR", "simple_lnRRv"),
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n
  )

  test_lnRR <- with(testing_data, log(A_mean/C_mean))

  test_lnRRv <- with(testing_data,
    ((C_sd^2) / (C_n * C_mean^2)) + 
    ((A_sd^2) / (A_n * A_mean^2))
  )

  expect_equal(res$simple_lnRR, test_lnRR, tolerance = 1e-6)
  expect_equal(res$simple_lnRRv, test_lnRRv, tolerance = 1e-6)
})


test_that("lnRR function works fine with 'main' effect size using Morris method", {
  # This is main effect of factor A
  res <- lnRR_main(
    data = testing_data,
    method = "morris",
    col_names = c("main_lnRR", "main_lnRRv"),
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  expect_equal(res$main_lnRR, testing_data$A_main_lnRR, tolerance = 1e-6)
  expect_equal(res$main_lnRRv, testing_data$A_main_lnRRv, tolerance = 1e-6)
})


test_that("lnRR function works fine with 'inter' effect size", {
  res <- lnRR_inter(
    data = testing_data,
    col_names = c("inter_lnRR", "inter_lnRRv"),
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  expect_equal(res$inter_lnRR, testing_data$AB_main_lnRR, tolerance = 1e-6)
  expect_equal(res$inter_lnRRv, testing_data$AB_main_lnRRv, tolerance = 1e-6)
})

#' Individual Effect: Log Coefficient Of Variation Ratio
#'
#' Computes the Log of the Coefficient of Variation Ratio between Factor A
#' and the Control treatment.
#'
#' See the package vignette for a detailed description of the formula.
#'
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param Ctrl_mean Mean outcome from the Control treatment
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_mean Mean outcome from the treatment
#' @param A_sd Standard deviation from the treatment
#' @param A_n Sample size from the treatment
#'
#' @inherit lnRR_ind return
#'
#' @references
#' Nakagawa, S., Poulin, R., Mengersen, K., Reinhold, K., Engqvist,
#'     L., Lagisz, M., & Senior, A. M. (2015). Meta‐analysis of variation: 
#'     ecological and evolutionary applications and beyond. Methods in
#'     Ecology and Evolution, 6(2), 143-152.
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @examples
#' data <- data.frame(
#'   study_id = 1:3,
#'   control_mean = c(8.5, 12.3, 6.8),
#'   control_sd = c(1.8, 2.9, 1.4),
#'   control_n = c(18, 24, 16),
#'   nutrient_mean = c(11.2, 16.7, 9.3),
#'   nutrient_sd = c(3.1, 4.8, 2.7),
#'   nutrient_n = c(19, 22, 17)
#' )
#' 
#' result <- lnCVR_ind(
#'   data = data,
#'   Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
#'   A_mean = "nutrient_mean", A_sd = "nutrient_sd", A_n = "nutrient_n"
#' )
#' 
#' @export
lnCVR_ind <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n
) {
  .assert_args(col_names, append, data)

  call_args <- as.list(match.call())[-1]

  lncvr_args <- .get_columns(call_args[.lnCVR_args$ind], data)

  df <- .compute_and_format(
    effsize_func = ".simple_lnCVR",
    effsize_args = lncvr_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' Main Effect: Log Coefficient Of Variation Ration
#' 
#' Computes the main effect of Factor A across levels of Factor B
#' in factorial experiments on the coefficient of variation.
#'
#' See the package vignette for a detailed description of the formula.
#'
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param Ctrl_mean Mean outcome from the Control treatment
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_mean Mean outcome from the treatment
#' @param A_sd Standard deviation from the treatment
#' @param A_n Sample size from the treatment
#' @param B_mean Mean outcome from the B treatment
#' @param B_sd Standard deviation from the B treatment 
#' @param B_n Sample size from the B treatment 
#' @param AB_mean Mean outcome from the interaction AxB treatment
#' @param AB_sd Standard deviation from the interaction AxB treatment
#' @param AB_n Sample size from the interaction AxB treatment
#'
#' @inherit lnRR_ind return
#' 
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @examples
#' data <- data.frame(
#'   study_id = 1:2,
#'   control_mean = c(14.2, 16.8), control_sd = c(2.8, 3.1), control_n = c(16, 14),
#'   irrigation_mean = c(19.5, 22.1), irrigation_sd = c(5.2, 5.8), irrigation_n = c(15, 16),
#'   co2_mean = c(16.8, 19.4), co2_sd = c(3.1, 3.6), co2_n = c(17, 13),
#'   irrigation_co2_mean = c(24.3, 27.9), irrigation_co2_sd = c(6.8, 7.4), irrigation_co2_n = c(14, 15)
#' )
#' 
#' result <- lnCVR_main(
#'   data = data,
#'   Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
#'   A_mean = "irrigation_mean", A_sd = "irrigation_sd", A_n = "irrigation_n",
#'   B_mean = "co2_mean", B_sd = "co2_sd", B_n = "co2_n", 
#'   AB_mean = "irrigation_co2_mean", AB_sd = "irrigation_co2_sd", AB_n = "irrigation_co2_n"
#' )
#'
#' @export
lnCVR_main <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  .assert_args(col_names, append, data)

  call_args <- as.list(match.call())[-1]

  lncvr_args <- .get_columns(call_args[.lnCVR_args$main], data)

  df <- .compute_and_format(
    effsize_func = ".main_lnCVR",
    effsize_args = lncvr_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' Interaction Effect: Log Coefficient of Variation Ratio
#'
#' Computes the interaction effect between Factors A and B
#' in factorial experiments on the coefficient of variation ratio.
#'
#' See the package vignette for a detailed description of the formula.
#'
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param Ctrl_mean Mean outcome from the Control treatment
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_mean Mean outcome from the treatment
#' @param A_sd Standard deviation from the treatment
#' @param A_n Sample size from the treatment
#' @param B_mean Mean outcome from the B treatment
#' @param B_sd Standard deviation from the B treatment 
#' @param B_n Sample size from the B treatment 
#' @param AB_mean Mean outcome from the interaction AxB treatment
#' @param AB_sd Standard deviation from the interaction AxB treatment
#' @param AB_n Sample size from the interaction AxB treatment
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @examples
#' # Interaction effect logCVR (Light x Nutrients)
#' data <- data.frame(
#'   study_id = 1:2,
#'   control_mean = c(7.3, 8.9),
#'   control_sd = c(1.4, 1.7),
#'   control_n = c(20, 18),
#'   light_mean = c(12.8, 14.2),
#'   light_sd = c(3.1, 3.5),
#'   light_n = c(19, 20),
#'   nutrients_mean = c(9.6, 11.1),
#'   nutrients_sd = c(1.9, 2.2),
#'   nutrients_n = c(21, 17),
#'   light_nutrients_mean = c(18.4, 20.7),
#'   light_nutrients_sd = c(4.8, 5.3),
#'   light_nutrients_n = c(18, 19)
#' )
#' 
#' result <- lnCVR_inter(
#'   data = data,
#'   Ctrl_mean = "control_mean",
#'   Ctrl_sd = "control_sd",
#'   Ctrl_n = "control_n",
#'   A_mean = "light_mean",
#'   A_sd = "light_sd",
#'   A_n = "light_n",
#'   B_mean = "nutrients_mean",
#'   B_sd = "nutrients_sd",
#'   B_n = "nutrients_n",
#'   AB_mean = "light_nutrients_mean",
#'   AB_sd = "light_nutrients_sd",
#'   AB_n = "light_nutrients_n"
#' )
#'
#' @export
lnCVR_inter <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  .assert_args(col_names, append, data)

  call_args <- as.list(match.call())[-1]

  lncvr_args <- .get_columns(call_args[.lnCVR_args$main], data)  # Same args as main

  df <- .compute_and_format(
    effsize_func = ".interaction_lnCVR",
    effsize_args = lncvr_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' @keywords internal
.lnCVR_args <- list(
  ind = c(
    "Ctrl_mean",
    "Ctrl_sd",
    "Ctrl_n",
    "A_mean",
    "A_sd",
    "A_n"
  ),
  main = c(
    "Ctrl_mean",
    "Ctrl_sd",
    "Ctrl_n",
    "A_mean",
    "A_sd",
    "A_n",
    "B_mean",
    "B_sd",
    "B_n",
    "AB_mean",
    "AB_sd",
    "AB_n"
  )
)

test_that("lnCVR works fine with 'ind' effects", {
  df <- data.frame(
    Ctrl_mean = 2.5,
    Ctrl_sd = 1.5,
    Ctrl_n = 10,
    A_mean = 4,
    A_sd = 4.3,
    A_n = 10
  )

  # Computed manually
  test_lnCVR <- 0.5831463
  test_lnCVRv <- 0.2626736

  # Compute with function
  res <- lnCVR_ind(
    data = df,
    col_names = c("simple_lnCVR", "simple_lnCVRv"),
    Ctrl_mean = Ctrl_mean,
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n
  )

  expect_equal(res$simple_lnCVR, test_lnCVR, tolerance = 1e-6)
  expect_equal(res$simple_lnCVRv, test_lnCVRv, tolerance = 1e-6)
})


test_that("lnCVR works fine with 'main' effects", {
  df <- data.frame(
    Ctrl_mean = 2,
    Ctrl_sd = 2,
    Ctrl_n = 10,
    A_mean = 4,
    A_sd = 4,
    A_n = 10,
    B_mean = 8,
    B_sd = 1,
    B_n = 10,
    AB_mean = 6,
    AB_sd = 3,
    AB_n = 10
  )

  # Computed manually
  test_lnCVR <- 0.69314718

  test_lnCVRv <- 0.1121962

  # Compute with function
  res <- lnCVR_main(
    data = df,
    col_names = c("main_lnCVR", "main_lnCVRv"),
    Ctrl_mean = Ctrl_mean,
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  expect_equal(res$main_lnCVR, test_lnCVR, tolerance = 1e-6)
  expect_equal(res$main_lnCVRv, test_lnCVRv, tolerance = 1e-6)
})


test_that("lnCVR works fine with 'inter' effects", {
  df <- data.frame(
    Ctrl_mean = 2,
    Ctrl_sd = 2,
    Ctrl_n = 10,
    A_mean = 4,
    A_sd = 4,
    A_n = 10,
    B_mean = 8,
    B_sd = 1,
    B_n = 10,
    AB_mean = 6,
    AB_sd = 3,
    AB_n = 10
  )

  # Computed manually
  test_lnCVR <- 1.38629436
  test_lnCVRv <- 0.448784722

  # Compute with function
  res <- lnCVR_inter(
    data = df,
    col_names = c("inter_lnCVR", "inter_lnCVRv"),
    Ctrl_mean = Ctrl_mean,
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  expect_equal(res$inter_lnCVR, test_lnCVR, tolerance = 1e-6)
  expect_equal(res$inter_lnCVRv, test_lnCVRv, tolerance = 1e-6)
})

#' Log Coefficient of Variation Ratio: Interaction Between Treatment and Time
#'
#' @inheritParams time_lnRR
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @references 
#'   Shinichi Nakagawa and Daniel Noble, personal communication.
#'
#' @examples
#' # Pre-post design for coefficient of variation changes over time (Disturbance experiment)
#' data <- data.frame(
#'   study_id = 1:2,
#'   pre_control_mean = c(12.8, 15.4), pre_control_sd = c(2.6, 3.1),
#'   post_control_mean = c(13.2, 15.9), post_control_sd = c(2.7, 3.2),
#'   control_n = c(20, 18),
#'   pre_disturbed_mean = c(12.9, 15.2), pre_disturbed_sd = c(2.5, 3.0),
#'   post_disturbed_mean = c(8.7, 10.1), post_disturbed_sd = c(3.8, 4.3),
#'   disturbed_n = c(19, 21)
#' )
#' 
#' result <- time_lnCVR(
#'   data = data,
#'   t0_Ctrl_mean = "pre_control_mean", t0_Ctrl_sd = "pre_control_sd",
#'   t1_Ctrl_mean = "post_control_mean", t1_Ctrl_sd = "post_control_sd",
#'   Ctrl_n = "control_n", Ctrl_cor = 0.8,
#'   t0_Exp_mean = "pre_disturbed_mean", t0_Exp_sd = "pre_disturbed_sd",
#'   t1_Exp_mean = "post_disturbed_mean", t1_Exp_sd = "post_disturbed_sd",
#'   Exp_n = "disturbed_n", Exp_cor = 0.5
#' )
#'
#' @export
time_lnCVR <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  t0_Ctrl_mean,
  t0_Ctrl_sd,
  t1_Ctrl_mean,
  t1_Ctrl_sd,
  Ctrl_n,
  Ctrl_cor,
  t0_Exp_mean,
  t0_Exp_sd,
  t1_Exp_mean,
  t1_Exp_sd,
  Exp_n,
  Exp_cor
) {
  .assert_args(col_names, append, data)
  .assert_cor_value(Ctrl_cor, data)
  .assert_cor_value(Exp_cor, data)

  call_args <- as.list(match.call())[-1]

  time_lncvr_func <- ".time_interaction_lnCVR"
  time_lncvr_args <-.get_columns(call_args[.time_lnCVR_requirements], data)

  time_lncvr_args$Ctrl_cor <- Ctrl_cor
  time_lncvr_args$Exp_cor <- Exp_cor

  df <- .compute_and_format(
    data = data,
    effsize_func = time_lncvr_func,
    effsize_args = time_lncvr_args,
    col_names = col_names,
    append = append
  )

  return(df)
}


.time_lnCVR_requirements <- c(
   "t0_Ctrl_mean",
   "t0_Ctrl_sd",
   "t1_Ctrl_mean",
   "t1_Ctrl_sd",
   "Ctrl_n",
   "t0_Exp_mean",
   "t0_Exp_sd",
   "t1_Exp_mean",
   "t1_Exp_sd",
   "Exp_n"
)


#' Log Coefficient of Variation Ratio: Interaction Between Experimental Treatment and Time
#' 
#' @inheritParams .time_interaction_lnRR
#'
#' @inherit lnRR_ind return
#'
#' @keywords internal
.time_interaction_lnCVR <- function(
   t0_Ctrl_mean,
   t0_Ctrl_sd,
   t1_Ctrl_mean,
   t1_Ctrl_sd,
   Ctrl_n,
   Ctrl_cor,
   t0_Exp_mean,
   t0_Exp_sd,
   t1_Exp_mean,
   t1_Exp_sd,
   Exp_n,
   Exp_cor
) {
  # First calculate the coefficients of variation
  t0_Ctrl_CV <- t0_Ctrl_sd / t0_Ctrl_mean
  t1_Ctrl_CV <- t1_Ctrl_sd / t1_Ctrl_mean
  t0_Exp_CV <- t0_Exp_sd / t0_Exp_mean
  t1_Exp_CV <- t1_Exp_sd / t1_Exp_mean

  # Calculate lnRR and lnVR for the sampling variance
  lnRR <- .time_interaction_lnRR(
    t0_Ctrl_mean = t0_Ctrl_mean,
    t0_Ctrl_sd = t0_Ctrl_sd, 
    t1_Ctrl_mean = t1_Ctrl_mean,
    t1_Ctrl_sd = t1_Ctrl_sd, 
    Ctrl_n = Ctrl_n,
    Ctrl_cor = Ctrl_cor,
    t0_Exp_mean = t0_Exp_mean,
    t0_Exp_sd = t0_Exp_sd,
    t1_Exp_mean = t1_Exp_mean,
    t1_Exp_sd = t1_Exp_sd,
    Exp_n = Exp_n,
    Exp_cor = Exp_cor
  )

  lnVR <- .time_interaction_lnVR(
    t0_Ctrl_sd = t0_Ctrl_sd, 
    t1_Ctrl_sd = t1_Ctrl_sd, 
    Ctrl_n = Ctrl_n,
    Ctrl_cor = Ctrl_cor,
    t0_Exp_sd = t0_Exp_sd,
    t1_Exp_sd = t1_Exp_sd,
    Exp_n = Exp_n,
    Exp_cor = Exp_cor
  )
  
  lnCVR <- log((t1_Exp_CV / t1_Ctrl_CV) / (t0_Exp_CV / t0_Ctrl_CV))

  lnCVRv <- lnRR[, 2] + lnVR[, 2]

  return(data.frame(lnCVR, lnCVRv))
}

test_that(".time_interaction_lnCVR works fine", {
  res <- .time_interaction_lnCVR(
    t0_Ctrl_mean = 10,
    t0_Ctrl_sd = 5,
    t1_Ctrl_mean = 14,
    t1_Ctrl_sd = 7,
    Ctrl_n = 10,
    Ctrl_cor = 0.5,
    t0_Exp_mean = 20,
    t0_Exp_sd = 5,
    t1_Exp_mean = 25,
    t1_Exp_sd = 2,
    Exp_n = 10,
    Exp_cor = 0.5
  )

  test_lnCVR <- -1.139434283
  test_lnCVRv <- 0.1965567

  expect_equal(res$lnCVR, test_lnCVR, tolerance = 1e-6)
  expect_equal(res$lnCVRv, test_lnCVRv, tolerance = 1e-6)
})


test_that(".time_interaction_lnCVR works fine", {
  df <- data.frame(
    t0_Ctrl_mean = 10,
    t0_Ctrl_sd = 5,
    t1_Ctrl_mean = 14,
    t1_Ctrl_sd = 7,
    Ctrl_n = 10,
    t0_Exp_mean = 20,
    t0_Exp_sd = 5,
    t1_Exp_mean = 25,
    t1_Exp_sd = 2,
    Exp_n = 10
  )

  res <- time_lnCVR(
    data = df,
    t0_Ctrl_mean = t0_Ctrl_mean,
    t0_Ctrl_sd = t0_Ctrl_sd,
    t1_Ctrl_mean = t1_Ctrl_mean,
    t1_Ctrl_sd = t1_Ctrl_sd,
    Ctrl_n = Ctrl_n,
    Ctrl_cor = 0.5,
    t0_Exp_mean = t0_Exp_mean,
    t0_Exp_sd = t0_Exp_sd,
    t1_Exp_mean = t1_Exp_mean,
    t1_Exp_sd = t1_Exp_sd,
    Exp_n = Exp_n,
    Exp_cor = 0.5
  )

  test_lnCVR <- -1.139434283
  test_lnCVRv <- 0.1965567

  expect_equal(res$yi, test_lnCVR, tolerance = 1e-6)
  expect_equal(res$vi, test_lnCVRv, tolerance = 1e-6)
})

#' Compute And Format
#' 
#' Compute effect size and do some pre-return tasks
#'
#' @param data Data frame to use
#' @param effsize_func Function used to compute the effect size
#' @param effsize_args Arguments needed for that function
#' @param col_names Vector of length 2 with the name of the columns 
#' @param append Logical. If append or not the effect sizes to the data
#'
#' @keywords internal
.compute_and_format <- function(
  effsize_func,
  effsize_args,
  data,
  col_names,
  append
) {
  checkmate::assert_choice(effsize_func, choices = c(
    ### lnRR ###
    ".simple_lnRR",
    ".main_lnRR_Nakagawa",
    ".main_lnRR_Morris",
    ".interaction_lnRR",
    ### lnVR ###
    ".simple_lnVR",
    ".main_lnVR",
    ".interaction_lnVR",
    ### lnCVR ###
    ".simple_lnCVR",
    ".main_lnCVR",
    ".interaction_lnCVR",
    ### SMD ###
    ".simple_SMD",
    ".main_SMD",
    ".interaction_SMD",
    ### Factor x Time Interactions ###
    ".time_interaction_lnRR",
    ".time_interaction_lnVR",
    ".time_interaction_lnCVR",
    ".time_interaction_SMD"
    )
  )
  df <- do.call(effsize_func, effsize_args)
  names(df) <- col_names

  if (.has_infinite(df)) {
    df <- .infinite_to_NA(df)
  }

  if (append) {
    df <- cbind(data, df)
  }

  return(df)
}


.get_columns <- function(columns_list, data) {
  # Find and extract the columns from columns_list in data

  # NAs are required columns missing
  if (any(is.na(names(columns_list)))) {
    fun_name <- as.character(sys.call(-1)[[1]])
    stop(sprintf("Some required numeric arguments are missing. See ?%s", fun_name),
         call. = FALSE)
  }
  
  return_cols <- list()

  for (arg in names(columns_list)) {
    col_name <- as.character(columns_list[[arg]])
  
    .assert_column_exists(col_name, data)
    .assert_is_numeric(col_name, data)
    .assert_no_NA(col_name, data)
    
    # If column is sample sizes (e.g., Ctrl_n)
    if (endsWith(arg, "_n")) {
      tryCatch(
        .assert_positive(col_name, data),
        error = function(e) stop(e$message, "\nSample sizes must be positive values.", call. = FALSE)
      )
    }

    return_cols[[arg]] <- data[[col_name]]
  }

  return(return_cols)
}


.assert_column_exists <- function(col_name, data) {
  if (!(col_name %in% names(data))) {
    stop(sprintf("the column %s doesn't exists.", col_name), call. = FALSE)
  }
}


.assert_is_numeric <- function(col_name, data) {
  column = data[[col_name]]
  if (!is.numeric(column)) {
    stop(sprintf("the column %s is not numeric.", col_name), call. = FALSE)
  }
}


.assert_no_NA <- function(col_name, data) {
  column = data[[col_name]]
  if (anyNA(column)) {
    stop(sprintf("the column %s has NAs.", col_name), call. = FALSE)
  }
}


.assert_positive <- function(col_name, data) {
  column = data[[col_name]]
  if (any(column <= 0)) {
    stop(sprintf("the column %s has zeros or negative values", col_name), call. = FALSE)
  }
}


.assert_args <- function(col_names, append, data) {
  checkmate::assert_character(col_names, len = 2)
  checkmate::assert_logical(append, len = 1)
  checkmate::assert_data_frame(data)
}


.assert_cor_value <- function(x, data) {
  # Check if x is a valid correlation value that ranges between -1 and 1
  if (!checkmate::test_numeric(x, lower = -1, upper = 1)) {
    stop(sprintf(
      "Correlation values must be between -1 and 1, but some values in %s are out of range.",
      deparse(substitute(x))
    ), call. = FALSE)
  }

  # Check if x is a number or a vector of length of data
  if (!(checkmate::test_numeric(x, len = nrow(data)) ||
        checkmate::test_numeric(x, len = 1))) {
    stop(sprintf(
      "length of %s must be 1 or equal to data, but is %d",
      deparse(substitute(x)), length(x)
    ), call. = FALSE)
  }
}


.has_infinite <- function(x) {
  return(any(is.infinite(unlist(x))))
}


.infinite_to_NA <- function(df) {
  df_replaced <- lapply(df, function(x) { 
           replace(x, is.infinite(x), NA)
  })
  warning("Some effect sizes resulted in infinite values (division by zero). These values were recoded as NA.",
          call. = FALSE)

  return(as.data.frame(df_replaced))
}

test_that(".assert_column_exists works", {
  foo <- data.frame(aaa = c(1:5),
		    bbb = c(1:5))

  expect_no_error(.assert_column_exists("aaa", foo))
  expect_error(.assert_column_exists("zzz", foo))
})


test_that(".assert_is_numeric works", {
  df <- data.frame(
      some_numbers = c(1:10),
      some_strings = LETTERS[1:10],
      some_factors <- as.factor(letters[1:10])
  )

  expect_no_error(.assert_is_numeric("some_numbers", df))  
  expect_error(.assert_is_numeric("some_strings", df))
  expect_error(.assert_is_numeric("some_factors", df))
})


test_that(".assert_no_NA throws error when NA is present", {
   df <- data.frame(good_col = c(10, 20, 30), bad_col = c(10, NA, 30))

   expect_no_error(.assert_no_NA("good_col", df))
   expect_error(.assert_no_NA("bad_col", df))
})


test_that(".assert_positive throws error when zeros or negatives are present", {
   df <- data.frame(
       good_col = c(10, 20, 30), 
       zero_col = c(0, 20, 30),
       negative_col = c(-1, 20, 30)
   )

   expect_no_error(.assert_positive("good_col", df))
   expect_error(.assert_positive("zero_col", df))
   expect_error(.assert_positive("negative_col", df))
})


test_that(".get_columns works by passing column name as symbols", {
  foo <- data.frame(aaa = c(1:5),
		    bbb = c(5:9))

  cols <- list(one_to_five = substitute(aaa))

  res <- .get_columns(cols, foo)

  expect_equal(res$one_to_five, c(1:5))
})


test_that(".get_columns throws errors when columns are not numeric", {
  foo <- data.frame(aaa = c(1:5),
		    ccc = LETTERS[1:5],
		    eee = rep(TRUE, 5))

  cols_chars <- list(some_chars = substitute(ccc))
  cols_logical <- list(some_logical = substitute(eee))

  expect_error(.get_columns(cols_chars, foo))
  expect_error(.get_columns(cols_logical, foo))
})


test_that(".get_columns throws errors when column does't exits", {
  foo <- data.frame(aaa = c(1:5),
		    ccc = LETTERS[1:5],
		    eee = rep(TRUE, 5))
  bad_colname <- list(some_chars = substitute(bbb))

  expect_error(.get_columns(bad_colname, foo))
})


test_that(".assert_cor_value throws errors when values are not between -1 and 1", {
  df <- data.frame(foo = c("a", "b", "c"))

  good_vector <- c(0, 0.2, -0.3)
  bad_vector <- c(0.2, 1, 3)

  expect_error(.assert_cor_value(bad_vector, df))
  expect_no_error(.assert_cor_value(good_vector, df))
})


test_that(".assert_cor_value throws error when length is not 1 or equal to len(data)", {
  df <- data.frame(foo = c("a", "b", "c"))

  good_vector <- c(0, 0.2, -0.3)
  bad_vector <- c(0.2, 1)
  another_good <- 0.5

  expect_error(.assert_cor_value(bad_vector, df))
  expect_no_error(.assert_cor_value(good_vector, df))
  expect_no_error(.assert_cor_value(another_good, df))
})


test_that(".has_infinite detects Inf and -Inf in dataframes", {
  good_df <- data.frame(foo = c(1, 2, 3))
  pos_inf <- data.frame(foo = c(1, Inf, 3))
  neg_inf <- data.frame(foo = c(1, 2, -Inf))

  expect_false(.has_infinite(good_df))
  expect_true(.has_infinite(pos_inf))
  expect_true(.has_infinite(neg_inf))
})


test_that(".infinite_to_NA transforms infinites into NAs", {
  df <- data.frame(foo = c(10, Inf, 10, -Inf))
  expected_df <- data.frame(foo = c(10, NA, 10, NA))
 
  expect_warning(.infinite_to_NA(df))
 
  # Avoid warning to store new_df. If not, a warning will arise in test
  suppressWarnings(new_df <- .infinite_to_NA(df))
  expect_equal(new_df, expected_df)
})


test_that(".compute_and_format warns about infinite values", {
  studies <- data.frame(
    Ctrl_mean = c(12.3, 0),
    Ctrl_sd   = c(2.1, 3.2),
    Ctrl_n    = c(22, 24),
    Fert_mean = c(18.5, 21.3),
    Fert_sd   = c(3.1, 4.1),
    Fert_n    = c(22, 25)
  )

  expect_warning(
    res <- lnRR_ind(
      data = studies,
      Ctrl_mean = "Ctrl_mean", Ctrl_sd = "Ctrl_sd", Ctrl_n = "Ctrl_n",
      A_mean = "Fert_mean", A_sd = "Fert_sd", A_n = "Fert_n"
    )
  )

  expect_true(any(is.na(res$yi)))
})


test_that(".get_columns throws error when required column is missing", {
  studies <- data.frame(
    Ctrl_mean = c(12.3, 0),
    Ctrl_sd   = c(2.1, 3.2),
    Ctrl_n    = c(22, 24),
    Fert_mean = c(18.5, 21.3),
    Fert_sd   = c(3.1, 4.1),
    Fert_n    = c(22, 25)
  )

  # Missing 'Ctrl_mean'
  expect_error(
    lnRR_ind(
      data = studies,
      Ctrl_sd = "Ctrl_sd", Ctrl_n = "Ctrl_n",
      A_mean = "Fert_mean", A_sd = "Fert_sd", A_n = "Fert_n"
    )
  )
})

#' Log of Variability Ratio: Interaction Between Treatment and Time
#'
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param t0_Ctrl_sd Standard deviation from the control group at time 0
#' @param t1_Ctrl_sd Standard deviation from the control group at time 1
#' @param Ctrl_n Sample size of the control group
#' @param Ctrl_cor Number or numeric vector. Correlation between the means of the control group at t0 and t1
#' @param t0_Exp_sd Standard deviation from the experimental group at time 0
#' @param t1_Exp_sd Standard deviation from the experimental group at time 1
#' @param Exp_n Sample size of the experimental group
#' @param Exp_cor Number or numeric vector. Correlation between the means of the experimental group at t0 and t1
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @references 
#'   Shinichi Nakagawa and Daniel Noble, personal communication.
#'
#' @examples
#' data <- data.frame(
#'   study_id = 1:2,
#'   pre_control_sd = c(2.1, 2.4),
#'   post_control_sd = c(2.2, 2.5),
#'   control_n = c(24, 19),
#'   pre_invaded_sd = c(2.0, 2.3),
#'   post_invaded_sd = c(4.1, 4.6),
#'   invaded_n = c(21, 22)
#' )
#' 
#' result <- time_lnVR(
#'   data = data,
#'   t0_Ctrl_sd = "pre_control_sd", t1_Ctrl_sd = "post_control_sd",
#'   Ctrl_n = "control_n", Ctrl_cor = 0.6,
#'   t0_Exp_sd = "pre_invaded_sd", t1_Exp_sd = "post_invaded_sd",
#'   Exp_n = "invaded_n", Exp_cor = 0.4
#' )
#'
#' @export
time_lnVR <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  t0_Ctrl_sd,
  t1_Ctrl_sd,
  Ctrl_n,
  Ctrl_cor,
  t0_Exp_sd,
  t1_Exp_sd,
  Exp_n,
  Exp_cor
) {
  .assert_args(col_names, append, data)
  .assert_cor_value(Ctrl_cor, data)
  .assert_cor_value(Exp_cor, data)

  call_args <- as.list(match.call())[-1]

  time_lnvr_func <- ".time_interaction_lnVR"
  time_lnvr_args <-.get_columns(call_args[.time_lnVR_requirements], data)

  time_lnvr_args$Ctrl_cor <- Ctrl_cor
  time_lnvr_args$Exp_cor <- Exp_cor

  df <- .compute_and_format(
    data = data,
    effsize_func = time_lnvr_func,
    effsize_args = time_lnvr_args,
    col_names = col_names,
    append = append
  )

  return(df)
}


.time_lnVR_requirements <- c(
   "t0_Ctrl_sd",
   "t1_Ctrl_sd",
   "Ctrl_n",
   "t0_Exp_sd",
   "t1_Exp_sd",
   "Exp_n"
)


#' Log of Variability Ratio: Interaction Between Experimental Treatment and Time
#'
#' @param t0_Ctrl_sd Standard deviation from the control group at time 0
#' @param t1_Ctrl_sd Standard deviation from the control group at time 1
#' @param Ctrl_n Sample size of the control group
#' @param Ctrl_cor Correlation between the means of the control group at t0 and t1
#' @param t0_Exp_sd Standard deviation from the experimental group at time 0
#' @param t1_Exp_sd Standard deviation from the experimental group at time 1
#' @param Exp_n Sample size of the experimental group
#' @param Exp_cor Correlation between the means of the experimental group at t0 and t1
#'
#' @keywords internal
.time_interaction_lnVR <- function(
   t0_Ctrl_sd,
   t1_Ctrl_sd,
   Ctrl_n,
   Ctrl_cor,
   t0_Exp_sd,
   t1_Exp_sd,
   Exp_n,
   Exp_cor
) {
  lnVR <- log((t1_Exp_sd / t1_Ctrl_sd) / (t0_Exp_sd / t0_Ctrl_sd))
  
  lnVRv <- ((1 - Exp_cor^2) / (Exp_n - 1)) +
        ((1 - Ctrl_cor^2) / (Ctrl_n - 1)) 

  return(data.frame(lnVR, lnVRv))
}

test_that(".time_interaction_lnVR is working fine", {
  res <- .time_interaction_lnVR(
    t0_Ctrl_sd = 2,
    t1_Ctrl_sd = 1,
    Ctrl_n = 10,
    Ctrl_cor = 0.5,
    t0_Exp_sd = 7,
    t1_Exp_sd = 10,
    Exp_n = 8,
    Exp_cor = 0.5
  )

  # Calculated manually
  test_lnVR <- 1.049822124
  test_lnVRv <- 0.19047619

  expect_equal(res$lnVR, test_lnVR, tolerance = 1e-6)
  expect_equal(res$lnVRv, test_lnVRv, tolerance = 1e-6)
})


test_that(".time__lnVR is working fine", {
  df <- data.frame(
    t0_Ctrl_sd = 2,
    t1_Ctrl_sd = 1,
    Ctrl_n = 10,
    t0_Exp_sd = 7,
    t1_Exp_sd = 10,
    Exp_n = 8
  )

  res <- time_lnVR(
    data = df,
    t0_Ctrl_sd = t0_Ctrl_sd,
    t1_Ctrl_sd = t1_Ctrl_sd,
    Ctrl_n = Ctrl_n,
    Ctrl_cor = 0.5,
    t0_Exp_sd = t0_Exp_sd,
    t1_Exp_sd = t1_Exp_sd,
    Exp_n = Exp_n,
    Exp_cor = 0.5
  )

  # Calculated manually
  test_lnVR <- 1.049822124
  test_lnVRv <- 0.19047619

  expect_equal(res$yi, test_lnVR, tolerance = 1e-6)
  expect_equal(res$vi, test_lnVRv, tolerance = 1e-6)
})

#' Standardized Mean Difference: Interaction Between Treatment and Time
#'
#' @inheritParams time_lnRR
#' @param hedges_correction Logical. Apply or not Hedges' correction for small-sample bias. Default is TRUE.
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @references 
#'   Shinichi Nakagawa and Daniel Noble, personal communication.
#'
#' @examples
#' # Pre-post design for standardized mean difference with time interaction (Conservation experiment)
#' data <- data.frame(
#'   study_id = 1:2,
#'   pre_control_mean = c(18.3, 21.7), pre_control_sd = c(4.1, 4.8),
#'   post_control_mean = c(18.8, 22.1), post_control_sd = c(4.2, 4.9),
#'   control_n = c(16, 14),
#'   pre_conservation_mean = c(18.1, 21.4), pre_conservation_sd = c(4.0, 4.7),
#'   post_conservation_mean = c(26.7, 31.2), post_conservation_sd = c(5.8, 6.4),
#'   conservation_n = c(15, 16)
#' )
#' 
#' result <- time_SMD(
#'   data = data,
#'   t0_Ctrl_mean = "pre_control_mean", t0_Ctrl_sd = "pre_control_sd",
#'   t1_Ctrl_mean = "post_control_mean", t1_Ctrl_sd = "post_control_sd",
#'   Ctrl_n = "control_n", Ctrl_cor = 0.9,
#'   t0_Exp_mean = "pre_conservation_mean", t0_Exp_sd = "pre_conservation_sd",
#'   t1_Exp_mean = "post_conservation_mean", t1_Exp_sd = "post_conservation_sd",
#'   Exp_n = "conservation_n", Exp_cor = 0.7,
#'   hedges_correction = TRUE
#' )
#' 
#' # Without Hedges' correction
#' result_no_hedges <- time_SMD(
#'   data = data,
#'   t0_Ctrl_mean = "pre_control_mean", t0_Ctrl_sd = "pre_control_sd",
#'   t1_Ctrl_mean = "post_control_mean", t1_Ctrl_sd = "post_control_sd",
#'   Ctrl_n = "control_n", Ctrl_cor = 0.9,
#'   t0_Exp_mean = "pre_conservation_mean", t0_Exp_sd = "pre_conservation_sd",
#'   t1_Exp_mean = "post_conservation_mean", t1_Exp_sd = "post_conservation_sd",
#'   Exp_n = "conservation_n", Exp_cor = 0.7,
#'   hedges_correction = FALSE
#' )
#' 
#' @export
time_SMD <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  hedges_correction = TRUE,
  t0_Ctrl_mean,
  t0_Ctrl_sd,
  t1_Ctrl_mean,
  t1_Ctrl_sd,
  Ctrl_n,
  Ctrl_cor,
  t0_Exp_mean,
  t0_Exp_sd,
  t1_Exp_mean,
  t1_Exp_sd,
  Exp_n,
  Exp_cor
) {
  .assert_args(col_names, append, data)
  .assert_cor_value(Ctrl_cor, data)
  .assert_cor_value(Exp_cor, data)

  call_args <- as.list(match.call())[-1]

  smd_func <- ".time_interaction_SMD"
  smd_args <-.get_columns(call_args[.time_lnRR_requirements], data)

  smd_args$Ctrl_cor <- Ctrl_cor
  smd_args$Exp_cor <- Exp_cor

  smd_args$hedges_correction <- hedges_correction

  df <- .compute_and_format(
    data = data,
    effsize_func = smd_func,
    effsize_args = smd_args,
    col_names = col_names,
    append = append
  )

  return(df)

}


#' Standardized Mean Difference for the interaction between Experimental Treatment and Time
#'
#' @inheritParams .time_interaction_lnRR
#' @param hedges_correction Logical. Apply or not Hedges' correction for small-sample bias. Default is TRUE.
#'
#' @references 
#'   Shinichi Nakagawa and Daniel Noble, personal communication.
#' 
#' @keywords internal
.time_interaction_SMD <- function(
  t0_Ctrl_mean,
  t0_Ctrl_sd,
  t1_Ctrl_mean,
  t1_Ctrl_sd,
  Ctrl_n,
  Ctrl_cor,
  t0_Exp_mean,
  t0_Exp_sd,
  t1_Exp_mean,
  t1_Exp_sd,
  Exp_n,
  Exp_cor,
  hedges_correction = TRUE
) {
  if (hedges_correction) {
    j <- .j_correction(Ctrl_n + Exp_n - 2)
  } else {
    j <- 1
  }

  pooled_sd <- .time_pooled_sd(
    t0_Ctrl_sd = t0_Ctrl_sd,
    t0_Exp_sd  = t0_Exp_sd,
    t1_Ctrl_sd = t1_Ctrl_sd,
    t1_Exp_sd  = t1_Exp_sd,
    Ctrl_n     = Ctrl_n,
    Exp_n      = Exp_n
  )

  d <- (((t1_Exp_mean - t1_Ctrl_mean) - (t0_Exp_mean - t0_Ctrl_mean)) / pooled_sd) * j
  v <- ((2 * (1 - Exp_cor)) / Exp_n) +
       ((2 * (1 - Ctrl_cor)) / Ctrl_n) +
       (d^2 / (2 * (Exp_n + Ctrl_n)))

  return(data.frame(yi = d, vi = v))
}


#' Pooled Standard Deviation for SMD in non-independent factorial
#'
#' @param t0_Ctrl_sd Standard deviation from Control treatment at time 0
#' @param t1_Ctrl_sd Standard deviation from Control treatment at time 1
#' @param Ctrl_n Sample size from Control treatment
#' @param t0_Exp_sd Standard deviation from Experimental treatment at time 0
#' @param t1_Exp_sd Standard deviation from Experimental treatment at time 1
#' @param Exp_n Sample size from Experimental treatment
#'
#' @keywords internal
.time_pooled_sd <- function(
  t0_Ctrl_sd,
  t1_Ctrl_sd,
  Ctrl_n,
  t0_Exp_sd,
  t1_Exp_sd,
  Exp_n
) {
  pooled_sd <- sqrt(
    (
      ((Exp_n - 1) * t0_Exp_sd^2) +
      ((Exp_n - 1) * t1_Exp_sd^2) +
      ((Ctrl_n - 1) * t0_Ctrl_sd^2) +
      ((Ctrl_n - 1) * t1_Ctrl_sd^2)
    ) /
    (
      (2 * (Exp_n + Ctrl_n - 2))
    )
  )

  return(pooled_sd)
}

test_that("The pooled standard deviation for the factor X time SMD is working fine", {
  pooled_sd <- .time_pooled_sd(
    t0_Ctrl_sd = 5,
    t1_Ctrl_sd = 6,
    Ctrl_n = 5,
    t0_Exp_sd = 2,
    t1_Exp_sd = 4,
    Exp_n = 5
  )
  
  # Computed manually
  test_pooled_sd <- 4.5

  expect_equal(pooled_sd, test_pooled_sd)
})


test_that("SMD of the interaction between Experimental Treatment and Time is working fine", {
  res <- .time_interaction_SMD(
    t0_Ctrl_mean = 10,
    t0_Ctrl_sd = 5, 
    t1_Ctrl_mean = 12,
    t1_Ctrl_sd = 6,
    Ctrl_n = 5,
    Ctrl_cor = 0.5,
    t0_Exp_mean = 12,
    t0_Exp_sd = 2,
    t1_Exp_mean = 20,
    t1_Exp_sd = 4,
    Exp_n = 5,
    Exp_cor = 0.5,
    hedges_correction = FALSE
  )

  # Computed manually 
  test_SMD <- 1.333333
  test_SMDv <- 0.488889

  expect_equal(res$yi, test_SMD, tolerance = 1e-6)
  expect_equal(res$vi, test_SMDv, tolerance = 1e-6)
})


test_that("SMD of the interaction between Experimental Treatment and Time is working fine", {
  df <- data.frame(
    t0_Ctrl_mean = 10,
    t0_Ctrl_sd = 5, 
    t1_Ctrl_mean = 12,
    t1_Ctrl_sd = 6,
    Ctrl_n = 5,
    t0_Exp_mean = 12,
    t0_Exp_sd = 2,
    t1_Exp_mean = 20,
    t1_Exp_sd = 4,
    Exp_n = 5
  )

  res <- time_SMD(
    data = df,
    hedges_correction = FALSE,
    t0_Ctrl_mean = t0_Ctrl_mean,
    t0_Ctrl_sd = t0_Ctrl_sd,
    t1_Ctrl_mean = t1_Ctrl_mean,
    t1_Ctrl_sd = t1_Ctrl_sd,
    Ctrl_n = Ctrl_n,
    Ctrl_cor = 0.5,
    t0_Exp_mean = t0_Exp_mean,
    t0_Exp_sd = t0_Exp_sd,
    t1_Exp_mean = t1_Exp_mean,
    t1_Exp_sd = t1_Exp_sd,
    Exp_n = Exp_n,
    Exp_cor = 0.5
  )

  # Computed manually 
  test_SMD <- 1.333333
  test_SMDv <- 0.488889

  expect_equal(res$yi, test_SMD, tolerance = 1e-6)
  expect_equal(res$vi, test_SMDv, tolerance = 1e-6)
})


test_that("tim_SMD works fine with Hedge's correction", {
  df <- data.frame(
    t0_Ctrl_mean = 10,
    t0_Ctrl_sd = 5, 
    t1_Ctrl_mean = 12,
    t1_Ctrl_sd = 6,
    Ctrl_n = 5,
    t0_Exp_mean = 12,
    t0_Exp_sd = 2,
    t1_Exp_mean = 20,
    t1_Exp_sd = 4,
    Exp_n = 5
  )

  res <- time_SMD(
    data = df,
    hedges_correction = TRUE,
    t0_Ctrl_mean = t0_Ctrl_mean,
    t0_Ctrl_sd = t0_Ctrl_sd,
    t1_Ctrl_mean = t1_Ctrl_mean,
    t1_Ctrl_sd = t1_Ctrl_sd,
    Ctrl_n = Ctrl_n,
    Ctrl_cor = 0.5,
    t0_Exp_mean = t0_Exp_mean,
    t0_Exp_sd = t0_Exp_sd,
    t1_Exp_mean = t1_Exp_mean,
    t1_Exp_sd = t1_Exp_sd,
    Exp_n = Exp_n,
    Exp_cor = 0.5
  )

  # Computed manually 
  test_SMD <- 1.204301
  test_SMDv <- 0.4725171

  expect_equal(res$yi, test_SMD, tolerance = 1e-6)
  expect_equal(res$vi, test_SMDv, tolerance = 1e-6)
})

#' @keywords internal
.simple_lnVR <- function(
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n
) {
  simple_lnVR <- log(A_sd / Ctrl_sd) + 
    (1 / (2 * (A_n - 1))) -
    (1 / (2 * (Ctrl_n - 1)))

  simple_lnVRv <- (1 / (2 * (A_n - 1))) + (1 / (2 * (Ctrl_n - 1)))

  return(data.frame(simple_lnVR, simple_lnVRv))
}


#' @keywords internal
.main_lnVR <- function(
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n,
  B_sd,
  B_n,
  AB_sd,
  AB_n
) {
  main_lnVR <- 0.5 * log((AB_sd * A_sd) / (B_sd * Ctrl_sd)) +
	  0.5 * (
            (1 / (2 * (AB_n - 1))) +
            (1 / (2 * (A_n - 1))) -
            (1 / (2 * (B_n - 1))) -
            (1 / (2 * (Ctrl_n - 1))) 
          )

  main_lnVRv <- 0.25 * (
	      (1 / (2 * (AB_n - 1))) +
              (1 / (2 * (A_n - 1))) +
              (1 / (2 * (B_n - 1))) +
              (1 / (2 * (Ctrl_n - 1))) 
            )

  return(data.frame(main_lnVR, main_lnVRv))
}


#' @keywords internal
.interaction_lnVR <- function(
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n,
  B_sd,
  B_n,
  AB_sd,
  AB_n
) {
  inter_lnVR <- log((AB_sd / B_sd) / (A_sd / Ctrl_sd)) +
            (1 / (2 * (AB_n - 1))) -
            (1 / (2 * (A_n - 1))) -
            (1 / (2 * (B_n - 1))) +
            (1 / (2 * (Ctrl_n - 1))) 

  inter_lnVRv <- (1 / (2 * (AB_n - 1))) +
            (1 / (2 * (A_n - 1))) +
            (1 / (2 * (B_n - 1))) +
            (1 / (2 * (Ctrl_n - 1)))

  return(data.frame(inter_lnVR, inter_lnVRv))
}

test_that("The simple lnVR is computed correctly", {
  C_sd <- 1.5
  C_n <- 10
  A_sd <- 2.5
  A_n <- 10

  test_lnVR <- log(A_sd / C_sd) + (1 / (2 * (A_n - 1))) - (1 / (2 * (C_n - 1)))
  test_lnVRv <- (1 / (2 * (A_n - 1))) + (1 / (2 * (C_n - 1)))

  res <- .simple_lnVR(
    Ctrl_sd = C_sd, 
    Ctrl_n = C_n,
    A_sd = A_sd,
    A_n = A_n
  )

  expect_equal(res$simple_lnVR, test_lnVR, tolerance = 1e-6)
  expect_equal(res$simple_lnVRv, test_lnVRv, tolerance = 1e-6)
})


test_that("The main lnVR is computed correctly", {
  df <- data.frame(
    Ctrl_sd = 2.0,
    Ctrl_n = 12,
    A_sd = 2.8,
    A_n = 12,
    B_sd = 2.4,
    B_n = 12,
    AB_sd = 3.6,
    AB_n = 12
  )

  test_main_lnVR <- 0.37096
  test_main_lnVRv <- 0.045454

  res <- with(df, .main_lnVR(
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_sd = A_sd,
    A_n = A_n,
    B_sd = B_sd,
    B_n = B_n,
    AB_sd = AB_sd,
    AB_n = AB_n
  ))

  expect_equal(res$main_lnVR, test_main_lnVR, tolerance = 1e-4)
  expect_equal(res$main_lnVRv, test_main_lnVRv, tolerance = 1e-4)
})


test_that("The interaction lnVR is computed correctly", {
  df <- data.frame(
    Ctrl_sd = 2.0,
    Ctrl_n = 12,
    A_sd = 2.8,
    A_n = 12,
    B_sd = 2.4,
    B_n = 12,
    AB_sd = 3.6,
    AB_n = 12
  )

  test_inter_lnVR <- 0.06899
  test_inter_lnVRv <- 0.18181

  res <- with(df, .interaction_lnVR(
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_sd = A_sd,
    A_n = A_n,
    B_sd = B_sd,
    B_n = B_n,
    AB_sd = AB_sd,
    AB_n = AB_n
  ))

  expect_equal(res$inter_lnVR, test_inter_lnVR, tolerance = 1e-4)
  expect_equal(res$inter_lnVRv, test_inter_lnVRv, tolerance = 1e-4)
})

#' Fake factorial dataset with pre-computed effect sizes
#'
#' This dataset contains fake data from 2×2 factorial experiments with four treatment groups:
#' Control (C), Factor A, Factor B, and the combination AB (A and B together). It includes means,
#' standard deviations, and sample sizes for each group, as well as pre-computed effect sizes:
#' log response ratio (lnRR), standardized mean difference (SMD), and their variances, for the main
#' effects of A, B, and the interaction (AB).
#'
#' @format A data frame with 10 rows and 24 columns:
#' \describe{
#'   \item{C_n, C_mean, C_sd}{Sample size, mean, and SD of the Control group}
#'   \item{A_n, A_mean, A_sd}{Sample size, mean, and SD of the A treatment}
#'   \item{B_n, B_mean, B_sd}{Sample size, mean, and SD of the B treatment}
#'   \item{AB_n, AB_mean, AB_sd}{Sample size, mean, and SD of the AB (A and B combined) treatment}
#'   \item{A_main_lnRR, A_main_lnRRv}{Log response ratio for main effect A and its variance}
#'   \item{B_main_lnRR, B_main_lnRRv}{Log response ratio for main effect B and its variance}
#'   \item{AB_main_lnRR, AB_main_lnRRv}{Log response ratio for the AB interaction and its variance}
#'   \item{A_main_SMD, A_main_SMDv}{Standardized mean difference (Hedges' g) for main effect A and its variance}
#'   \item{B_main_SMD, B_main_SMDv}{Standardized mean difference for main effect B and its variance}
#'   \item{AB_main_SMD, AB_main_SMDv}{Standardized mean difference for the AB interaction and its variance}
#' }
#'
#' @details
#' This dataset is intended for validating functions that compute or test effect sizes
#' in meta-analysis workflows involving factorial designs.
#'
#' @source Fake.
#' 
#' @keywords internal
"testing_data"

#' Simple effect: Standardized Mean Difference 
#'
#' Computes the individual or simple effect of Factor A over the Control. 
#' 
#' It is the classic Standardized Mean Difference (SMD), which can also be computed
#' with metafor's `escalc()` function using `measure = "SMD"`.
#'
#' See the package vignette for a detailed description of the formula.
#' 
#' @inheritParams lnRR_ind
#' @param hedges_correction Boolean. If TRUE correct for small-sample bias. Default is TRUE.
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @references
#'   Gurevitch, J., Morrison, J. A., & Hedges, L. V. (2000). The interaction
#'     between competition and predation: a meta-analysis of field experiments.
#'     The American Naturalist, 155(4), 435-453.
#' 
#'   Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
#'     Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
#'     effects of enemies and mutualists on plant performance: a meta‐analysis. 
#'     Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
#'
#' @examples
#' data <- data.frame(
#'   study_id = 1:3,
#'   control_mean = c(45.2, 52.8, 38.9),
#'   control_sd = c(8.1, 11.2, 7.3),
#'   control_n = c(18, 23, 16),
#'   pollinator_exclusion_mean = c(28.7, 35.4, 22.1),
#'   pollinator_exclusion_sd = c(6.8, 9.1, 5.9),
#'   pollinator_exclusion_n = c(20, 22, 18)
#' )
#' 
#' # With Hedges' correction (default)
#' result <- SMD_ind(
#'   data = data,
#'   Ctrl_mean = "control_mean",
#'   Ctrl_sd = "control_sd",
#'   Ctrl_n = "control_n",
#'   A_mean = "pollinator_exclusion_mean",
#'   A_sd = "pollinator_exclusion_sd",
#'   A_n = "pollinator_exclusion_n",
#'   hedges_correction = TRUE
#' )
#' 
#' # Without Hedges' correction
#' result_no_hedges <- SMD_ind(
#'   data = data,
#'   Ctrl_mean = "control_mean",
#'   Ctrl_sd = "control_sd",
#'   Ctrl_n = "control_n",
#'   A_mean = "pollinator_exclusion_mean",
#'   A_sd = "pollinator_exclusion_sd",
#'   A_n = "pollinator_exclusion_n",
#'   hedges_correction = FALSE
#' )
#'
#' @export
SMD_ind <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  hedges_correction = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n
) {
  .assert_args(col_names, append, data)
  checkmate::assert_logical(hedges_correction, len = 1)

  call_args <- as.list(match.call())[-1]

  smd_args <- .get_columns(call_args[.SMD_args$ind], data)
  smd_args$hedges_correction <- hedges_correction

  df <- .compute_and_format(
    effsize_func = ".simple_SMD",
    effsize_args = smd_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' Main effect: Standardized Mean Difference 
#' 
#' Computes the main effect of Factor A across levels of Factor B, analogous
#' to the main effect in a factorial ANOVA. 
#'
#' See the package vignette for a detailed description of the formula.
#'
#' @inheritParams lnRR_main
#' @param hedges_correction Boolean. If TRUE correct for small-sample bias. Default is TRUE.
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#' 
#' @references 
#'   Gurevitch, J., Morrison, J. A., & Hedges, L. V. (2000). The interaction
#'     between competition and predation: a meta-analysis of field experiments.
#'     The American Naturalist, 155(4), 435-453.
#' 
#'   Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
#'     Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
#'     effects of enemies and mutualists on plant performance: a meta‐analysis. 
#'     Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
#'
#' @examples
#' # Main effect of Mycorrhiza in 2x2 factorial design (AMF x Phosphorus)
#' data <- data.frame(
#'   study_id = 1:2,
#'   control_mean = c(12.4, 15.1), control_sd = c(2.8, 3.2), control_n = c(16, 14),
#'   mycorrhizae_mean = c(18.7, 21.3), mycorrhizae_sd = c(3.4, 3.9), mycorrhizae_n = c(15, 16),
#'   phosphorus_mean = c(14.9, 17.8), phosphorus_sd = c(3.1, 3.6), phosphorus_n = c(17, 13),
#'   myco_phos_mean = c(22.1, 25.4), myco_phos_sd = c(4.2, 4.8), myco_phos_n = c(14, 15)
#' )
#' 
#' result <- SMD_main(
#'   data = data,
#'   Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
#'   A_mean = "mycorrhizae_mean", A_sd = "mycorrhizae_sd", A_n = "mycorrhizae_n",
#'   B_mean = "phosphorus_mean", B_sd = "phosphorus_sd", B_n = "phosphorus_n",
#'   AB_mean = "myco_phos_mean", AB_sd = "myco_phos_sd", AB_n = "myco_phos_n"
#' )
#'
#' @export
SMD_main <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  hedges_correction = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  .assert_args(col_names, append, data)
  checkmate::assert_logical(hedges_correction, len = 1)

  call_args <- as.list(match.call())[-1]

  smd_args <- .get_columns(call_args[.SMD_args$main], data)
  smd_args$hedges_correction <- hedges_correction

  df <- .compute_and_format(
    effsize_func = ".main_SMD",
    effsize_args = smd_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' Interaction effect: Standardized mean difference 
#' 
#' Computes the interaction effect between factors A and B in factorial
#' data.
#'
#' See the package vignette for a detailed description of the formula.
#' 
#' @inheritParams lnRR_inter
#' @param hedges_correction Logical. Apply or not Hedges' correction for small-sample bias. Default is TRUE
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#' 
#' @references 
#'   Gurevitch, J., Morrison, J. A., & Hedges, L. V. (2000). The interaction
#'     between competition and predation: a meta-analysis of field experiments.
#'     The American Naturalist, 155(4), 435-453.
#'
#'   Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
#'     Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
#'     effects of enemies and mutualists on plant performance: a meta‐analysis. 
#'     Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
#'
#' @examples
#' data <- data.frame(
#'   study_id = 1:2,
#'   control_mean = c(24.8, 27.2), control_sd = c(4.1, 4.6), control_n = c(18, 16),
#'   salinity_mean = c(19.3, 21.7), salinity_sd = c(3.8, 4.2), salinity_n = c(17, 18),
#'   temperature_mean = c(28.9, 31.4), temperature_sd = c(4.7, 5.1), temperature_n = c(19, 15),
#'   salt_temp_mean = c(15.2, 17.8), salt_temp_sd = c(3.1, 3.5), salt_temp_n = c(16, 17)
#' )
#' 
#' result <- SMD_inter(
#'   data = data,
#'   Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
#'   A_mean = "salinity_mean", A_sd = "salinity_sd", A_n = "salinity_n",
#'   B_mean = "temperature_mean", B_sd = "temperature_sd", B_n = "temperature_n",
#'   AB_mean = "salt_temp_mean", AB_sd = "salt_temp_sd", AB_n = "salt_temp_n"
#' )
#'
#' @export
SMD_inter <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  hedges_correction = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  .assert_args(col_names, append, data)
  checkmate::assert_logical(hedges_correction, len = 1)

  call_args <- as.list(match.call())[-1]

  smd_args <- .get_columns(call_args[.SMD_args$main], data)  # Same arguments as main
  smd_args$hedges_correction <- hedges_correction

  df <- .compute_and_format(
    effsize_func = ".interaction_SMD",
    effsize_args = smd_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' @keywords internal
.SMD_args <- list(
  ind = c(
    "Ctrl_mean",
    "Ctrl_sd",
    "Ctrl_n",
    "A_mean",
    "A_sd",
    "A_n"
  ),
  main = c(
    "Ctrl_mean",
    "Ctrl_sd",
    "Ctrl_n",
    "A_mean",
    "A_sd",
    "A_n",
    "B_mean",
    "B_sd",
    "B_n",
    "AB_mean",
    "AB_sd",
    "AB_n"
  )
)

test_that("SMD function works for 'ind' Cohen's d", {
  # Use same data from the Cohen's d test above
  df <- data.frame(
    Ctrl_mean = 2.5,
    Ctrl_sd = 1.5,
    Ctrl_n = 10,
    A_mean = 4,
    A_sd = 4.3,
    A_n = 10
  )

  # Computed with metafor::escalc
  test_SMD <- 0.4658026
  test_SMDv <- 0.2054243

  res <- SMD_ind(
    data = df,
    append = FALSE,
    col_names = c("simple_SMD", "simple_SMDv"),
    hedges_correction = FALSE,
    Ctrl_mean = Ctrl_mean,
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n
  )

  expect_equal(res$simple_SMD, test_SMD, tolerance = 1e-6)
  expect_equal(res$simple_SMDv, test_SMDv, tolerance = 1e-6)
})


test_that("SMD function works for 'ind' Hedges' g", {
  # Use same data from the Hedges' g test above
  df <- data.frame(
    Ctrl_mean = 2.5,
    Ctrl_sd = 1.5,
    Ctrl_n = 10,
    A_mean = 4,
    A_sd = 4.3,
    A_n = 10
  )

  # Computed with metafor::escalc
  test_SMD <- 0.44601
  test_SMDv <- 0.20497

  res <- SMD_ind(
    data = df,
    append = FALSE,
    col_names = c("simple_SMD", "simple_SMDv"),
    hedges_correction = TRUE,
    Ctrl_mean = Ctrl_mean,
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n
  )

  expect_equal(res$simple_SMD, test_SMD, tolerance = 1e-3)
  expect_equal(res$simple_SMDv, test_SMDv, tolerance = 1e-3)
})


test_that("SMD function computes fine the main Cohen's d", {
  # This computes the main SMD for Factor A
  res_A <- SMD_main(
    data = testing_data,
    col_names = c("main_SMD", "main_SMDv"),
    hedges_correction = FALSE,
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  expect_equal(res_A$main_SMD, testing_data$A_main_SMD, tolerance = 1e-6)
  expect_equal(res_A$main_SMDv, testing_data$A_main_SMDv, tolerance = 1e-6)
})


test_that("SMD function computes fine the interaction Cohen's d", {
  # This computes the main SMD for Factor A
  res <- SMD_inter(
    data = testing_data,
    col_names = c("inter_SMD", "inter_SMDv"),
    hedges_correction = FALSE,
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  expect_equal(res$inter_SMD, testing_data$AB_main_SMD, tolerance = 1e-6)
  expect_equal(res$inter_SMDv, testing_data$AB_main_SMDv, tolerance = 1e-6)
})


test_that("main_SMD works fine without Hedge's correction (Cohen's d)", {
  foo_data <- data.frame(
      Ctrl_mean = c(10),
      Ctrl_sd = c(5),
      Ctrl_n = c(10),
      A_mean = c(12),
      A_sd = c(5),
      A_n = c(9),
      B_mean = c(6),
      B_sd = c(4),
      B_n = c(10),
      AB_mean = c(12),
      AB_sd = c(5),
      AB_n = c(10)
  )

  res <- SMD_main(
    data = foo_data,
    hedges_correction = FALSE,
    Ctrl_mean = "Ctrl_mean",
    Ctrl_sd = "Ctrl_sd",
    Ctrl_n = "Ctrl_n",
    A_mean = "A_mean",
    A_sd = "A_sd",
    A_n = "A_n",
    B_mean = "B_mean",
    B_sd = "B_sd",
    B_n = "B_n",
    AB_mean = "AB_mean",
    AB_sd = "AB_sd",
    AB_n = "AB_n"
  )

  yi_expected <- 0.8398153
  vi_expected <- 0.1050383

  expect_equal(res$yi, yi_expected, tolerance = 1e-6)
  expect_equal(res$vi, vi_expected, tolerance = 1e-6)
})


test_that("SMD_main works fine with Hedge's correction", {
  foo_data <- data.frame(
      Ctrl_mean = c(10),
      Ctrl_sd = c(5),
      Ctrl_n = c(10),
      A_mean = c(12),
      A_sd = c(5),
      A_n = c(9),
      B_mean = c(6),
      B_sd = c(4),
      B_n = c(10),
      AB_mean = c(12),
      AB_sd = c(5),
      AB_n = c(10)
  )

  res <- SMD_main(
    data = foo_data,
    hedges_correction = TRUE,
    Ctrl_mean = "Ctrl_mean",
    Ctrl_sd = "Ctrl_sd",
    Ctrl_n = "Ctrl_n",
    A_mean = "A_mean",
    A_sd = "A_sd",
    A_n = "A_n",
    B_mean = "B_mean",
    B_sd = "B_sd",
    B_n = "B_n",
    AB_mean = "AB_mean",
    AB_sd = "AB_sd",
    AB_n = "AB_n"
  )

  yi_expected <- 0.8216898
  vi_expected <- 0.1049418

  expect_equal(res$yi, yi_expected, tolerance = 1e-6)
  expect_equal(res$vi, vi_expected, tolerance = 1e-6)
})


test_that("SMD_inter works fine with Hedge's correction", {
  foo_data <- data.frame(
      Ctrl_mean = c(10),
      Ctrl_sd = c(5),
      Ctrl_n = c(10),
      A_mean = c(12),
      A_sd = c(5),
      A_n = c(9),
      B_mean = c(6),
      B_sd = c(4),
      B_n = c(10),
      AB_mean = c(12),
      AB_sd = c(5),
      AB_n = c(10)
  )

  res <- SMD_inter(
    data = foo_data,
    hedges_correction = TRUE,
    Ctrl_mean = "Ctrl_mean",
    Ctrl_sd = "Ctrl_sd",
    Ctrl_n = "Ctrl_n",
    A_mean = "A_mean",
    A_sd = "A_sd",
    A_n = "A_n",
    B_mean = "B_mean",
    B_sd = "B_sd",
    B_n = "B_n",
    AB_mean = "AB_mean",
    AB_sd = "AB_sd",
    AB_n = "AB_n"
  )

  yi_expected <- 0.8216898
  vi_expected <- 0.4197672

  expect_equal(res$yi, yi_expected, tolerance = 1e-6)
  expect_equal(res$vi, vi_expected, tolerance = 1e-6)
})

#' @inherit SMD_ind
#' @keywords internal
.simple_SMD <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  hedges_correction = TRUE
) {
  # Compute the effect size using correction for small-sample bias if needed.
  # Equation from Gurevitch et al. 2000 and Morris et al. 2007
  if (hedges_correction) {
    j <- .j_correction(A_n + Ctrl_n - 2)
  } else {
    j <- 1
  }

  pooled_sd <- .pooled_sd(
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_sd = A_sd,
    A_n = A_n
  )

  d <- ((A_mean - Ctrl_mean) / pooled_sd) * j
  v <- 1/A_n + 1/Ctrl_n + (d^2 / (2 * (A_n + Ctrl_n)))

  return(data.frame(simple_SMD = d, simple_SMDv = v))
}

#' @inherit SMD_main
#' @keywords internal
.main_SMD <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n,
  hedges_correction = TRUE
) {
  # Overral effect size of factor A
  if (hedges_correction) {
    j <- .j_correction(A_n + B_n + AB_n + Ctrl_n - 4)
  } else {
    j <- 1
  }

  pooled_sd <- .pooled_sd(
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_sd = A_sd,
    A_n = A_n,
    B_sd = B_sd,
    B_n = B_n,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  d_A <- (((A_mean + AB_mean) - (B_mean + Ctrl_mean)) / (2 * pooled_sd)) * j

  # Sampling variance. Formula from Gurevitch et al. 2000.
  # The 1/4 is missing in Morris et al 2007 appendix, but can be found in 
  # their Matlab code (https://dx.doi.org/10.6084/m9.figshare.c.3299699)
  v_A <- (1/4) * (
    1/A_n + 1/B_n + 1/AB_n + 1/Ctrl_n + 
    (d_A^2 / (2 * (A_n + B_n + AB_n + Ctrl_n)))
  )

  return(data.frame(main_SMD = d_A, main_SMDv = v_A))
}


#' @inherit SMD_inter
#' @keywords internal
.interaction_SMD <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean, 
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n,
  hedges_correction = TRUE
) {
  if (hedges_correction) {
    j <- .j_correction(A_n + B_n + AB_n + Ctrl_n - 4)
  } else {
    j <- 1
  }

  pooled_sd <- .pooled_sd(
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_sd = A_sd,
    A_n = A_n,
    B_sd = B_sd,
    B_n = B_n,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  d_Inter <- (((AB_mean - B_mean) - (A_mean - Ctrl_mean)) / pooled_sd) * j

  v <-  1/A_n + 1/B_n + 1/AB_n + 1/Ctrl_n + 
    (d_Inter^2 / (2 * (A_n + B_n + AB_n + Ctrl_n)))

  return(data.frame(inter_SMD = d_Inter, inter_SMDv = v))
}


#' Pooled Standard Deviation for SMD in factorial experiments
#'
#' Compute the pooled standard deviation for SMD. It computes the pooled SD
#' for 2 or 4 groups depending on the arguments passed. 
#' Simple SMD has only 2 groups, while main and interactions had 4 groups.
#'
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_sd Standard deviation from the A treatment
#' @param A_n Sample size from the A treatment
#' @param B_sd Standard deviation from the B treatment 
#' @param B_n Sample size from the B treatment 
#' @param AB_sd Standard deviation from the interaction AxB treatment
#' @param AB_n Sample size from the interaction AxB treatment
#'
#' @references
#'   Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
#'     Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
#'     effects of enemies and mutualists on plant performance: a meta‐analysis. 
#'     Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
#'
#' @keywords internal
.pooled_sd <- function(
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n,
  B_sd = NULL,
  B_n = NULL,
  AB_n = NULL,
  AB_sd = NULL
) {
  # If only Ctrl and A are passed, set all the other params to 0.
  # This way the pooled sd is for only 2 groups.
  if (is.null(B_sd) && is.null(AB_sd)) {
    n_groups <- 2
    B_sd <- B_n <- AB_n <- AB_sd <- 0
  } else {
    n_groups <- 4
  }

  # Equation B.2 from Morris et al. appendix B
  pooled_sd <- sqrt(
    (
      ((A_n - 1) * A_sd^2) + 
      ((B_n - 1) * B_sd^2) + 
      ((AB_n - 1) * AB_sd^2) +
      ((Ctrl_n - 1) * Ctrl_sd^2)
    ) /
    (A_n + B_n + AB_n + Ctrl_n - n_groups)
  )

  return(pooled_sd)
}


#' Correction for small-sample bias
#' 
#' Small-sample bias correction for Hedges' g
#'
#' @param m Degrees of freedom. They change between individual and main effects.
#' 
#' @keywords internal
.j_correction <- function(m) {
  if (any(m <= 0)) {
    stop(
      "Degrees of freedom `m` must be > 0 for the Hedges correction. ",
      "Check that all group sample sizes are >= 2.",
      call. = FALSE
    )
  }
  j <- 1 - (3 / ((4 * m) - 1))
  return(j)
}

data(testing_data)

test_that("Pooled sampling variance is computed correctly", {
  test_S1 <- 8.276473   # Manually calculated
  res_S1 <- .pooled_sd(
    Ctrl_sd = 5,
    Ctrl_n = 5,
    A_sd = 10,
    A_n = 5,
    B_sd = 10,
    B_n = 5,
    AB_sd = 7,
    AB_n = 5
  )

  test_S2 <- 5.0249  # Page 227 from The Handbook of Meta Analysis (Borensetin et al. 2009)
  res_S2 <- .pooled_sd(
    Ctrl_sd = 4.5,
    Ctrl_n = 50,
    A_sd = 5.5,
    A_n = 50
  )

  expect_equal(res_S1, test_S1, tolerance = 1e-6)
  expect_equal(res_S2, test_S2, tolerance = 1e-4)

})


test_that(".j_correction is working fine", {
  test_j <- 0.9923  # From page 227 of The Handbook of Meta Analysis (Borenstein et al. 2009)
  res_j <- .j_correction(50 + 50 - 2)

  expect_equal(res_j, test_j, tolerance = 1e-4)
})


test_that("Simple SMD (Cohen's d) is working fine", {
  Ctrl_mean <- 2.5
  Ctrl_sd <- 1.5
  Ctrl_n <- 10
  A_mean <- 4
  A_sd <- 4.3
  A_n <- 10

  # Computed with metafor::escalc
  test_SMD <- 0.4658026
  test_SMDv <- 0.2054243

  res <- .simple_SMD(
    Ctrl_mean = Ctrl_mean,
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    hedges_correction = FALSE
  )

  expect_equal(res$simple_SMD, test_SMD, tolerance = 1e-6)
  expect_equal(res$simple_SMDv, test_SMDv, tolerance = 1e-6)
})


test_that("Simple SMD (Hedges' g) is working fine", {
  Ctrl_mean <- 2.5
  Ctrl_sd <- 1.5
  Ctrl_n <- 10
  A_mean <- 4
  A_sd <- 4.3
  A_n <- 10

  # Computed with metafor::escalc
  test_SMD <- 0.44601
  test_SMDv <- 0.20497

  res <- .simple_SMD(
    Ctrl_mean = Ctrl_mean,
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    hedges_correction = TRUE
  )

  expect_equal(res$simple_SMD, test_SMD, tolerance = 1e-3)
  expect_equal(res$simple_SMDv, test_SMDv, tolerance = 1e-3)
})


test_that("Main SMD (Cohen's d) is correctly computed", {
  # This computes the main SMD for Factor A
  res_A <- with(testing_data, .main_SMD(
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n,
    hedges_correction = FALSE
  ))

  expect_equal(res_A$main_SMD, testing_data$A_main_SMD, tolerance = 1e-6)
  expect_equal(res_A$main_SMDv, testing_data$A_main_SMDv, tolerance = 1e-6)
})



test_that("The interaction SMD (Cohen's d) is correctly computed", {
  # This computes the main SMD for Factor A
  res <- with(testing_data, .interaction_SMD(
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n,
    hedges_correction = FALSE
  ))

  expect_equal(res$inter_SMD, testing_data$AB_main_SMD, tolerance = 1e-6)
  expect_equal(res$inter_SMDv, testing_data$AB_main_SMDv, tolerance = 1e-6)
})

#' @keywords internal
.simple_lnRR <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n
) {
  simple_lnRR <- log(A_mean / Ctrl_mean)

  simple_lnRRv <- Ctrl_sd^2 / (Ctrl_n * Ctrl_mean^2) + A_sd^2 / (A_n * A_mean^2)

  return(data.frame(simple_lnRR, simple_lnRRv))
}


#' @keywords internal
.main_lnRR_Morris <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean, 
  B_sd, 
  B_n,
  AB_mean, 
  AB_sd,
  AB_n
) {
  # From Morris et al. 2007, formulas B.9 and B.10 from Appendix B:
  main_lnRR <- log(A_mean + AB_mean) - log(Ctrl_mean + B_mean)

  main_lnRRv <- (1 / (A_mean    + AB_mean))^2 * (A_sd^2    / A_n    + AB_sd^2 / AB_n) +
                (1 / (Ctrl_mean + B_mean ))^2 * (Ctrl_sd^2 / Ctrl_n + B_sd^2  / B_n)
  
  return(data.frame(main_lnRR, main_lnRRv)) 
}


# Main lnRR but using Shinichi Nakagawa's derivation.
# It use geometric mean instead of arithmetic.

#' @keywords internal
.main_lnRR_Nakagawa <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd, 
  A_n,
  B_mean,
  B_sd, 
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  main_lnRR <- 0.5 * log((A_mean * AB_mean) / (Ctrl_mean * B_mean))

  main_lnRRv <- 0.25 * (
    (AB_sd^2   / (AB_n   * AB_mean^2)) +
    (A_sd^2    / (A_n    * A_mean^2)) +
    (B_sd^2    / (B_n    * B_mean^2)) +
    (Ctrl_sd^2 / (Ctrl_n * Ctrl_mean^2))
  )

  return(data.frame(main_lnRR, main_lnRRv)) 
}	


#' @keywords internal
.interaction_lnRR <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  # From Morris et al. 2007, formulas B.11 and B.12 from Appendix B:
  inter_lnRR <- log(AB_mean / B_mean) - log(A_mean / Ctrl_mean)

  inter_lnRRv <- (AB_sd^2   / (AB_mean^2   * AB_n  )) +
                 (A_sd^2    / (A_mean^2    * A_n   )) +
                 (B_sd^2    / (B_mean^2    * B_n   )) +
                 (Ctrl_sd^2 / (Ctrl_mean^2 * Ctrl_n)) 

  return(data.frame(inter_lnRR, inter_lnRRv))
}

data(testing_data)

test_that("Simple lnRR is correctly computed", {
  res <- with(testing_data, .simple_lnRR(
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n
  ))

  test_lnRR <- with(testing_data, log(A_mean/C_mean))

  test_lnRRv <- with(testing_data,
    ((C_sd^2) / (C_n * C_mean^2)) + 
    ((A_sd^2) / (A_n * A_mean^2))
  )

  expect_equal(res$simple_lnRR, test_lnRR, tolerance = 1e-6)
  expect_equal(res$simple_lnRRv, test_lnRRv, tolerance = 1e-6)
})


test_that("The main lnRR is correctly computed with Morris method", {
  res <- with(testing_data, .main_lnRR_Morris(
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  ))

  expect_equal(res$main_lnRR, testing_data$A_main_lnRR, tolerance = 1e-6)
  expect_equal(res$main_lnRRv, testing_data$A_main_lnRRv, tolerance = 1e-6)
})


test_that("The main lnRR using Nakagawa's method is correctly computed", {
  res <- .main_lnRR_Nakagawa(
    Ctrl_mean = 10,
    Ctrl_sd = 2,
    Ctrl_n = 10,
    A_mean = 12,
    A_sd = 2.5,
    A_n = 10,
    B_mean = 8,
    B_sd = 1.5,
    B_n = 10,
    AB_mean = 15,
    AB_sd = 3,
    AB_n = 10
  )

  # Manually computed for testing
  test_lnRR <- 0.405465108
  test_lnRRv <- 0.003963975

  expect_equal(res$main_lnRR, test_lnRR, tolerance = 1e-6)
  expect_equal(res$main_lnRRv, test_lnRRv, tolerance = 1e-6)
})


test_that("The interaction lnRR is correctly computed", {
  res <- with(testing_data, .interaction_lnRR(
    Ctrl_mean = C_mean,
    Ctrl_sd = C_sd,
    Ctrl_n = C_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  ))

  expect_equal(res$inter_lnRR, testing_data$AB_main_lnRR, tolerance = 1e-6)
  expect_equal(res$inter_lnRRv, testing_data$AB_main_lnRRv, tolerance = 1e-6)
})

#' Individual effect: Log of Variability Ratio
#'
#' Computes the Log of the Variability Ratio between
#' a Factor A and the Control treatment in factorial experiments.
#'
#' See the package vignette for a detailed description of the formula.
#'
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_sd Standard deviation from the treatment
#' @param A_n Sample size from the treatment
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @references
#' Nakagawa, S., Poulin, R., Mengersen, K., Reinhold, K., Engqvist,
#'     L., Lagisz, M., & Senior, A. M. (2015). Meta‐analysis of variation: 
#'     ecological and evolutionary applications and beyond. Methods in
#'     Ecology and Evolution, 6(2), 143-152.
#'
#' @examples
#' # Example focusing on variability differences (Herbivory effect)
#' data <- data.frame(
#'   study_id = 1:3,
#'   control_sd = c(2.1, 1.8, 2.5),
#'   control_n = c(20, 22, 18),
#'   herbivory_sd = c(3.2, 2.9, 3.8),
#'   herbivory_n = c(21, 20, 19)
#' )
#' 
#' result <- lnVR_ind(
#'   data = data,
#'   Ctrl_sd = "control_sd",
#'   Ctrl_n = "control_n",
#'   A_sd = "herbivory_sd", 
#'   A_n = "herbivory_n"
#' )
#'
#' @export
lnVR_ind <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n
) {
  .assert_args(col_names, append, data)

  call_args <- as.list(match.call())[-1]

  lnvr_args <- .get_columns(call_args[.lnVR_args$ind], data)

  df <- .compute_and_format(
    effsize_func = ".simple_lnVR",
    effsize_args = lnvr_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' Main Effect: Log of the Variability Ratio
#'
#' Computes the overral log of the variability ratio for Factor A
#' across levels of Factor B. 
#'
#' See the package vignette for a detailed description of the formula.
#' 
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_sd Standard deviation from the A treatment
#' @param A_n Sample size from the A treatment
#' @param B_sd Standard deviation from the B treatment 
#' @param B_n Sample size from the B treatment 
#' @param AB_sd Standard deviation from the interaction AxB treatment
#' @param AB_n Sample size from the interaction AxB treatment
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @examples
#' # Example for main effect in 2x2 factorial focusing on variability (Fire x Grazing)
#' data <- data.frame(
#'   study_id = 1:2,
#'   control_sd = c(2.0, 2.3), control_n = c(20, 18),
#'   fire_sd = c(2.8, 3.1), fire_n = c(19, 20),
#'   grazing_sd = c(2.2, 2.5), grazing_n = c(21, 17),
#'   fire_grazing_sd = c(3.5, 3.8), fire_grazing_n = c(18, 19)
#' )
#' 
#' result <- lnVR_main(
#'   data = data,
#'   Ctrl_sd = "control_sd", Ctrl_n = "control_n",
#'   A_sd = "fire_sd", A_n = "fire_n",
#'   B_sd = "grazing_sd", B_n = "grazing_n",
#'   AB_sd = "fire_grazing_sd", AB_n = "fire_grazing_n"
#' )
#'
#' @export
lnVR_main <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n,
  B_sd,
  B_n,
  AB_sd,
  AB_n
) {
  .assert_args(col_names, append, data)

  call_args <- as.list(match.call())[-1]

  lnvr_args <- .get_columns(call_args[.lnVR_args$main], data)

  df <- .compute_and_format(
    effsize_func = ".main_lnVR",
    effsize_args = lnvr_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' Interaction effect: Log Variability Ratio
#'
#' Computes the interaction of Factors A and B measured as 
#' the log of the variability ratio.
#'
#' See the package vignette for a detailed description of the formula.
#'
#' @param data Data frame containing the variables used.
#' @param col_names Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.
#' @param append Logical. Append the results to \code{data}. Default is TRUE
#' @param Ctrl_sd Standard deviation from the control treatment
#' @param Ctrl_n Sample size from the control treatment
#' @param A_sd Standard deviation from the A treatment
#' @param A_n Sample size from the A treatment
#' @param B_sd Standard deviation from the B treatment 
#' @param B_n Sample size from the B treatment 
#' @param AB_sd Standard deviation from the interaction AxB treatment
#' @param AB_n Sample size from the interaction AxB treatment
#'
#' @inherit lnRR_ind return
#'
#' @author Facundo Decunta - fdecunta@agro.uba.ar
#'
#' @examples
#' # Example for interaction effect in 2x2 factorial focusing on variability (Drought x Temperature)
#' data <- data.frame(
#'   study_id = 1:2,
#'   control_sd = c(1.8, 2.1), control_n = c(22, 19),
#'   drought_sd = c(2.6, 2.9), drought_n = c(20, 21),
#'   temperature_sd = c(2.0, 2.3), temperature_n = c(21, 18),
#'   drought_temp_sd = c(3.2, 3.6), drought_temp_n = c(19, 20)
#' )
#' 
#' result <- lnVR_inter(
#'   data = data,
#'   Ctrl_sd = "control_sd", Ctrl_n = "control_n",
#'   A_sd = "drought_sd", A_n = "drought_n", 
#'   B_sd = "temperature_sd", B_n = "temperature_n",
#'   AB_sd = "drought_temp_sd", AB_n = "drought_temp_n"
#' )
#'
#' @export
lnVR_inter <- function(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n,
  B_sd,
  B_n,
  AB_sd,
  AB_n
) {
  .assert_args(col_names, append, data)

  call_args <- as.list(match.call())[-1]

  lnvr_args <- .get_columns(call_args[.lnVR_args$main], data)  # Same args as main

  df <- .compute_and_format(
    effsize_func = ".interaction_lnVR",
    effsize_args = lnvr_args,
    data = data,
    col_names = col_names,
    append = append
  )

  return(df)
}


#' Required columns for computing lnVR
#'
#' 'inter' use the same arguments than 'main',
#  so there only 'main'
#'
#' @keywords internal
.lnVR_args <- list(
  ind = c(
    "Ctrl_sd",
    "Ctrl_n",
    "A_sd",
    "A_n"
  ),
  main = c(
    "Ctrl_sd",
    "Ctrl_n",
    "A_sd",
    "A_n",
    "B_sd",
    "B_n",
    "AB_sd",
    "AB_n"
  )
)

test_that("lnVR works fine for 'ind' effect size", {
  C_sd <- 1.5
  C_n <- 10
  A_sd <- 2.5
  A_n <- 10

  df <- data.frame(
    C_sd = C_sd,
    C_n = C_n,
    A_sd = A_sd,
    A_n = A_n
  )

  test_lnVR <- log(A_sd / C_sd) + (1 / (2 * (A_n - 1))) - (1 / (2 * (C_n - 1)))
  test_lnVRv <- (1 / (2 * (A_n - 1))) + (1 / (2 * (C_n - 1)))

  res <- lnVR_ind(
    data = df,
    col_names = c("simple_lnVR", "simple_lnVRv"),
    Ctrl_sd = C_sd, 
    Ctrl_n = C_n,
    A_sd = A_sd,
    A_n = A_n
  )

  expect_equal(res$simple_lnVR, test_lnVR, tolerance = 1e-6)
  expect_equal(res$simple_lnVRv, test_lnVRv, tolerance = 1e-6)
})


test_that("lnVR() works fine with 'main'", {
  df <- data.frame(
    Ctrl_sd = 1.5,
    Ctrl_n = 10,
    A_sd = 2.5,
    A_n = 10,
    B_sd = 3,
    B_n = 10,
    AB_sd = 4,
    AB_n = 10
  )

  # Main effect for factor A. Computed by hand
  main_lnVR <- 0.39925
  main_lnVRv <- 0.05556

  res <- lnVR_main(
    data = df,
    col_names = c("main_lnVR", "main_lnVRv"),
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_sd = A_sd,
    A_n = A_n,
    B_sd = B_sd,
    B_n = B_n,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  expect_equal(res$main_lnVR, main_lnVR, tolerance = 1e-4)
  expect_equal(res$main_lnVRv, main_lnVRv, tolerance = 1e-4)
})


test_that("lnVR works fine with 'inter'", {
  df <- data.frame(
    Ctrl_sd = 1.5,
    Ctrl_n = 10,
    A_sd = 2.5,
    A_n = 10,
    B_sd = 3,
    B_n = 10,
    AB_sd = 4,
    AB_n = 10
  )

  # Main effect for factor A. Computed by hand
  inter_lnVR <- -0.22314
  inter_lnVRv <- 0.22222

  res <- lnVR_inter(
    data = df,
    col_names = c("inter_lnVR", "inter_lnVRv"),
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_sd = A_sd,
    A_n = A_n,
    B_sd = B_sd,
    B_n = B_n,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  expect_equal(res$inter_lnVR, inter_lnVR, tolerance = 1e-4)
  expect_equal(res$inter_lnVRv, inter_lnVRv, tolerance = 1e-4)
})

#' @keywords internal
.simple_lnCVR <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n
) {
  # First compute the coefficients of variation 
  Ctrl_CV <- Ctrl_sd / Ctrl_mean
  A_CV <- A_sd / A_mean

  simple_lnCVR <- log(A_CV / Ctrl_CV) +
    (1 / (2 * (A_n - 1))) -
    (1 / (2 * (Ctrl_n - 1))) 

  # Assumes no correlation between mean and variance (see Nakagawa et al. 2015)
  simple_lnCVRv <- (Ctrl_sd^2 / (Ctrl_n * Ctrl_mean^2)) + (1 / (2 * (Ctrl_n - 1))) + 
    (A_sd^2 / (A_n * A_mean^2)) + (1 / (2 * (A_n - 1)))

  return(data.frame(simple_lnCVR, simple_lnCVRv))
}


#' @keywords internal
.main_lnCVR <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  # First compute the coefficients of variation 
  Ctrl_CV <- Ctrl_sd / Ctrl_mean
  A_CV <- A_sd / A_mean
  B_CV <- B_sd / B_mean
  AB_CV <- AB_sd / AB_mean

  # Also need lnRR and lnVR for the sampling variance
  main_lnRR <- .main_lnRR_Nakagawa(
    Ctrl_mean = Ctrl_mean, 
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  main_lnVR <- .main_lnVR(
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_sd = A_sd,
    A_n = A_n,
    B_sd = B_sd,
    B_n = B_n,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  # Assume no correlation between mean and variance (see Nakagawa et al. 2015)
  main_lnCVR <- 0.5 * log((AB_CV * A_CV) / (B_CV * Ctrl_CV)) +
    0.5 * (
           (1 / (2 * (AB_n - 1))) +
           (1 / (2 * (A_n - 1))) -
           (1 / (2 * (B_n - 1))) -
           (1 / (2 * (Ctrl_n - 1)))
    )

  # This is the sum of the variances of each
  # Uses '[, 2]' becuase the second column is their sampling variance
  main_lnCVRv <- main_lnRR[, 2] + main_lnVR[, 2]

  return(data.frame(main_lnCVR, main_lnCVRv))
}


#' @keywords internal
.interaction_lnCVR <- function(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
) {
  # First compute the coefficients of variation 
  Ctrl_CV <- Ctrl_sd / Ctrl_mean
  A_CV <- A_sd / A_mean
  B_CV <- B_sd / B_mean
  AB_CV <- AB_sd / AB_mean

  # Also need lnRR and lnVR for the sampling variance
  inter_lnRR <- .interaction_lnRR(
    Ctrl_mean = Ctrl_mean, 
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  inter_lnVR <- .interaction_lnVR(
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_sd = A_sd,
    A_n = A_n,
    B_sd = B_sd,
    B_n = B_n,
    AB_sd = AB_sd,
    AB_n = AB_n
  )

  inter_lnCVR <- log((AB_CV / B_CV) / (A_CV / Ctrl_CV)) +
    (
           (1 / (2 * (AB_n - 1))) -
           (1 / (2 * (A_n - 1))) -
           (1 / (2 * (B_n - 1))) +
           (1 / (2 * (Ctrl_n - 1)))
    )
  

  # This is the sum of the variances of each
  # Uses '[, 2]' becuase the second column is their sampling variance
  inter_lnCVRv <- inter_lnRR[, 2] + inter_lnVR[, 2]

  return(data.frame(inter_lnCVR, inter_lnCVRv))
}

test_that("Simple lnCVR is computed correctly", {
  Ctrl_mean <- 2.5
  Ctrl_sd <- 1.5
  Ctrl_n <- 10
  A_mean <- 4
  A_sd <- 4.3
  A_n <- 10

  # Computed manually
  test_lnCVR <- 0.5831463
  test_lnCVRv <- 0.2626736

  # Compute with function
  res <- .simple_lnCVR(
    Ctrl_mean = Ctrl_mean,
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n
  )

  expect_equal(res$simple_lnCVR, test_lnCVR, tolerance = 1e-6)
  expect_equal(res$simple_lnCVRv, test_lnCVRv, tolerance = 1e-6)
})


test_that("The main lnCVR is computed correctly", {
  df <- data.frame(
    Ctrl_mean = 10,
    Ctrl_sd = 2.0,
    Ctrl_n = 12,
    A_mean = 14,
    A_sd = 2.8,
    A_n = 12,
    B_mean = 12,
    B_sd = 2.4,
    B_n = 12,
    AB_mean = 20,
    AB_sd = 3.6,
    AB_n = 12
  )

  test_main_lnCVR <- -0.05268
  test_main_lnCVRv <- 0.04863

  res <- with(df, .main_lnCVR(
    Ctrl_mean = Ctrl_mean,                      
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  ))
 
  expect_equal(res$main_lnCVR, test_main_lnCVR, tolerance = 1e-4)
  expect_equal(res$main_lnCVRv, test_main_lnCVRv, tolerance = 1e-4)
})


test_that("The interaciont lnCVR is computed correctly", {
  df <- data.frame(
    Ctrl_mean = 10,
    Ctrl_sd = 2.0,
    Ctrl_n = 12,
    A_mean = 14,
    A_sd = 2.8,
    A_n = 12,
    B_mean = 12,
    B_sd = 2.4,
    B_n = 12,
    AB_mean = 20,
    AB_sd = 3.6,
    AB_n = 12
  )

  test_inter_lnCVR <- -0.10536
  test_inter_lnCVRv <- 0.19451

  res <- with(df, .interaction_lnCVR(
    Ctrl_mean = Ctrl_mean,                      
    Ctrl_sd = Ctrl_sd,
    Ctrl_n = Ctrl_n,
    A_mean = A_mean,
    A_sd = A_sd,
    A_n = A_n,
    B_mean = B_mean,
    B_sd = B_sd,
    B_n = B_n,
    AB_mean = AB_mean,
    AB_sd = AB_sd,
    AB_n = AB_n
  ))
 
  expect_equal(res$inter_lnCVR, test_inter_lnCVR, tolerance = 1e-4)
  expect_equal(res$inter_lnCVRv, test_inter_lnCVRv, tolerance = 1e-4)
})

---
title: "Effect Size Calculations with minter"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{minter Documentation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Overview

The **minter** package provides tools for calculating effect sizes used in meta-analyses, with specialized support for factorial experimental designs and repeated measures studies. The package offers a consistent interface for computing various effect size metrics.

## Key Features

**minter** implements two primary categories of effect size calculations:

### 1. Factorial Design Effect Sizes
For experimental designs with two factors (2×2 designs), supporting three types of comparisons:

- **Individual/Simple effects**: Direct comparisons between single treatments
- **Main effects**: Average effect of one factor across levels of another factor  
- **Interaction effects**: How the effect of one factor depends on the level of another factor

### 2. Repeated Measures Effect Sizes
For longitudinal studies examining treatment × time interactions in before-after or time-series designs.

## Quick Navigation

### Factorial Design Functions

#### Standardized Mean Difference (SMD)
- [`SMD_ind`](#smd_ind) - Individual/simple effect
- [`SMD_main`](#smd_main) - Main effect  
- [`SMD_inter`](#smd_inter) - Interaction effect

#### Log Response Ratio (lnRR)
- [`lnRR_ind`](#lnrr_ind) - Individual/simple effect
- [`lnRR_main`](#lnrr_main) - Main effect
- [`lnRR_inter`](#lnrr_inter) - Interaction effect

#### Log Variation Ratio (lnVR)
- [`lnVR_ind`](#lnvr_ind) - Individual/simple effect
- [`lnVR_main`](#lnvr_main) - Main effect
- [`lnVR_inter`](#lnvr_inter) - Interaction effect

#### Log Coefficient of Variation Ratio (lnCVR)
- [`lnCVR_ind`](#lncvr_ind) - Individual/simple effect
- [`lnCVR_main`](#lncvr_main) - Main effect
- [`lnCVR_inter`](#lncvr_inter) - Interaction effect

### Repeated Measures Functions
- [`time_SMD`](#time_smd) - Treatment × time interaction (SMD)
- [`time_lnRR`](#time_lnrr) - Treatment × time interaction (lnRR)
- [`time_lnVR`](#time_lnvr) - Treatment × time interaction (lnVR)
- [`time_lnCVR`](#time_lncvr) - Treatment × time interaction (lnCVR)

---

## Factorial Design Effect Sizes

These functions compute effect sizes for standard 2×2 factorial designs where two factors (A and B) are manipulated, creating four treatment groups: Control, A alone, B alone, and AB combined.

### Background

Effect sizes for factorial meta-analysis were first introduced by Gurevitch et al. (2000), who presented methods for estimating main and interaction effects analogous to factorial ANOVA. Originally developed for Standardized Mean Difference, these approaches have been extended to other effect size families including response ratios and variability measures.

Recent developments by Shinichi Nakagawa and Daniel Noble have expanded these methods to include lnVR and lnCVR effect sizes, providing researchers with comprehensive tools for meta-analyzing factorial experiments.

### Acknowledgments

We thank **Shinichi Nakagawa** and **Daniel Noble** for their theoretical contributions to factorial meta-analysis methodology and for generously sharing unpublished formulas.

---

## Standardized Mean Difference (SMD) {#smd}

The Standardized Mean Difference measures the magnitude of treatment effects in standard deviation units, making it comparable across studies with different measurement scales.

### Individual Effect: `SMD_ind()` {#smd_ind}

Computes the individual or simple effect of Factor A compared to Control. This is equivalent to the classic SMD that can be computed using metafor's `escalc()` function with `measure = "SMD"`.

**Formula:**
$$d_{ind} = \frac{\bar{X}_A - \bar{X}_{Ctrl}}{S_{pooled}} \cdot J(m)$$

where the pooled standard deviation is:
$$S_{pooled} = \sqrt{\frac{(n_A-1)sd_A^2 + (n_{Ctrl}-1)sd_{Ctrl}^2}{n_A + n_{Ctrl} - 2}}$$

**Small-sample bias correction:**
The $J(m)$ correction (Hedges correction) is applied by default but can be disabled:
$$J(m) = 1 - \frac{3}{4m-1}$$ 
where $m = n_A + n_{Ctrl} - 2$

**Sampling variance:**
$$var(d_{ind}) = \frac{1}{n_A} + \frac{1}{n_{Ctrl}} + \frac{d^2}{2(n_A + n_{Ctrl})}$$

### Main Effect: `SMD_main()` {#smd_main}

Computes the main effect of Factor A averaged across levels of Factor B, analogous to main effects in factorial ANOVA.

**Formula:**
$$d_{main} = \frac{(\bar{X}_A + \bar{X}_{AB}) - (\bar{X}_{B} + \bar{X}_{Ctrl})}{2S_{pooled}} \cdot J(m)$$

**Pooled standard deviation (four groups):**
$$S_{pooled} = \sqrt{\frac{(n_A-1)sd_A^2 + (n_B-1)sd_B^2 + (n_{AB}-1)sd_{AB}^2 + (n_{Ctrl}-1)sd_{Ctrl}^2}{n_A + n_B + n_{AB} + n_{Ctrl} - 4}}$$

**Degrees of freedom:** $m = n_A + n_B + n_{AB} + n_{Ctrl} - 4$

**Sampling variance:**
$$var(d_{main}) = \frac{1}{4} \left(\frac{1}{n_A} + \frac{1}{n_B} + \frac{1}{n_{AB}} + \frac{1}{n_{Ctrl}} + \frac{d_{main}^2}{2(n_A + n_B + n_{AB} + n_{Ctrl})}\right)$$

### Interaction Effect: `SMD_inter()` {#smd_inter}

Computes the interaction between factors A and B, measuring how the effect of A depends on the level of B.

**Formula:**
$$d_{inter} = \frac{(\bar{X}_{AB} - \bar{X}_B) - (\bar{X}_A - \bar{X}_{Ctrl})}{S_{pooled}} \cdot J(m)$$

Uses the same four-group pooled standard deviation as the main effect.

**Sampling variance:**
$$var(d_{inter}) = \frac{1}{n_A} + \frac{1}{n_B} + \frac{1}{n_{AB}} + \frac{1}{n_{Ctrl}} + \frac{d_{inter}^2}{2(n_A + n_B + n_{AB} + n_{Ctrl})}$$

---

## Log Response Ratio (lnRR) {#lnrr}

The Log Response Ratio measures proportional changes between treatments, providing intuitive interpretation.

### Individual Effect: `lnRR_ind()` {#lnrr_ind}

**Formula:**
$$\ln RR_{ind} = \ln\left(\frac{\bar{X}_A}{\bar{X}_{Ctrl}}\right)$$

**Sampling variance:**
$$var(\ln RR_{ind}) = \frac{sd_A^2}{n_A\bar{X}_A^2} + \frac{sd_{Ctrl}^2}{n_{Ctrl}\bar{X}_{Ctrl}^2}$$

### Main Effect: `lnRR_main()` {#lnrr_main}

**Formula:**
$$\ln RR_{main} = \ln\left(\frac{\bar{X}_A + \bar{X}_{AB}}{\bar{X}_{Ctrl} + \bar{X}_B}\right)$$

**Sampling variance:**
$$var(\ln RR_{main}) = \left(\frac{1}{\bar{X}_A + \bar{X}_{AB}}\right)^2 \left(\frac{sd_A^2}{n_A} + \frac{sd_{AB}^2}{n_{AB}}\right) + \left(\frac{1}{\bar{X}_{Ctrl} + \bar{X}_B}\right)^2 \left(\frac{sd_{Ctrl}^2}{n_{Ctrl}} + \frac{sd_B^2}{n_B}\right)$$

### Interaction Effect: `lnRR_inter()` {#lnrr_inter}

**Formula:**
$$\ln RR_{inter} = \ln\left(\frac{\bar{X}_{AB}}{\bar{X}_B}\right) - \ln\left(\frac{\bar{X}_A}{\bar{X}_{Ctrl}}\right)$$

**Sampling variance:**
$$var(\ln RR_{inter}) = \frac{sd_{AB}^2}{n_{AB}\bar{X}_{AB}^2} + \frac{sd_A^2}{n_A\bar{X}_A^2} + \frac{sd_B^2}{n_B\bar{X}_B^2} + \frac{sd_{Ctrl}^2}{n_{Ctrl}\bar{X}_{Ctrl}^2}$$

---

## Log Variation Ratio (lnVR) {#lnvr}

The Log Variation Ratio compares variability between treatments, providing insights into how treatments affect response consistency.

### Individual Effect: `lnVR_ind()` {#lnvr_ind}

**Formula:**
$$\ln VR_{ind} = \ln\left(\frac{sd_{A}}{sd_{Ctrl}}\right) + \frac{1}{2(n_{A} - 1)} - \frac{1}{2(n_{Ctrl} - 1)}$$

**Sampling variance:**
$$var(\ln VR_{ind}) = \frac{1}{2(n_{A} - 1)} + \frac{1}{2(n_{Ctrl} - 1)}$$

### Main Effect: `lnVR_main()` {#lnvr_main}

**Formula:**
$$\ln VR_{main} = \frac{1}{2} \ln\left( \frac{sd_{AB} \cdot sd_{A}}{sd_{B} \cdot sd_{Ctrl}} \right) + \frac{1}{2} \left( \frac{1}{2(n_{AB} - 1)} + \frac{1}{2(n_{A} - 1)} - \frac{1}{2(n_{B} - 1)} - \frac{1}{2(n_{Ctrl} - 1)} \right)$$

**Sampling variance:**
$$var(\ln VR_{main}) = \frac{1}{4} \left( \frac{1}{2(n_{AB} - 1)} + \frac{1}{2(n_{A} - 1)} + \frac{1}{2(n_{B} - 1)} + \frac{1}{2(n_{Ctrl} - 1)} \right)$$

### Interaction Effect: `lnVR_inter()` {#lnvr_inter}

**Formula:**
$$\ln VR_{inter} = \ln\left( \frac{sd_{AB} / sd_{B}}{sd_{A} / sd_{Ctrl}} \right) + \frac{1}{2(n_{AB} - 1)} - \frac{1}{2(n_{A} - 1)} - \frac{1}{2(n_{B} - 1)} + \frac{1}{2(n_{Ctrl} - 1)}$$

**Sampling variance:**
$$var(\ln VR_{inter}) = \frac{1}{2(n_{AB} - 1)} + \frac{1}{2(n_{A} - 1)} + \frac{1}{2(n_{B} - 1)} + \frac{1}{2(n_{Ctrl} - 1)}$$

---

## Log Coefficient of Variation Ratio (lnCVR) {#lncvr}

The Log Coefficient of Variation Ratio combines information about both mean responses and variability by comparing coefficients of variation ($CV = sd / \bar{x}$).

### Individual Effect: `lnCVR_ind()` {#lncvr_ind}

**Formula:**
$$\ln CVR_{ind} = \ln\left( \frac{CV_{A}}{CV_{Ctrl}} \right) + \frac{1}{2(n_{A} - 1)} - \frac{1}{2(n_{Ctrl} - 1)}$$

**Sampling variance:**
$var(\ln CVR_{ind}) = \frac{sd_{Ctrl}^2}{n_{Ctrl}\bar{X}_{Ctrl}^2} + \frac{1}{2(n_{Ctrl} - 1)} + \frac{sd_A^2}{n_A\bar{X}_A^2} + \frac{1}{2(n_A - 1)}$

This assumes no correlation between mean and variance (Nakagawa et al. 2015) and is computed as the sum of lnRR and lnVR variances.

### Main Effect: `lnCVR_main()` {#lncvr_main}

**Formula:**
$$\ln CVR_{main} = \frac{1}{2} \ln\left( \frac{CV_{AB} \cdot CV_{A}}{CV_{B} \cdot CV_{Ctrl}} \right) + \frac{1}{2} \left( \frac{1}{2(n_{AB} - 1)} + \frac{1}{2(n_{A} - 1)} - \frac{1}{2(n_{B} - 1)} - \frac{1}{2(n_{Ctrl} - 1)} \right)$$

**Sampling variance:**
$var(\ln CVR_{main}) = var(\ln RR_{main}) + var(\ln VR_{main})$

This follows the assumption of no correlation between mean and variance.

### Interaction Effect: `lnCVR_inter()` {#lncvr_inter}

**Formula:**
$$\ln CVR_{inter} = \ln\left( \frac{CV_{AB} / CV_{B}}{CV_{A} / CV_{Ctrl}} \right) + \frac{1}{2} \left( \frac{1}{2(n_{AB} - 1)} - \frac{1}{2(n_{A} - 1)} + \frac{1}{2(n_{B} - 1)} - \frac{1}{2(n_{Ctrl} - 1)} \right)$$

**Sampling variance:**
$var(\ln CVR_{inter}) = var(\ln RR_{inter}) + var(\ln VR_{inter})$

This follows the assumption of no correlation between mean and variance.

---

## Repeated Measures Effect Sizes

These functions compute effect sizes for treatment × time interactions in longitudinal studies, comparing how experimental and control groups change over time.

### Treatment × Time SMD: `time_SMD()` {#time_smd}

Computes the standardized mean difference for the interaction between experimental treatment and time.

**Formula:**
$$d = \frac{((\bar{X}_{t1,Exp} - \bar{X}_{t1,Ctrl}) - (\bar{X}_{t0,Exp} - \bar{X}_{t0,Ctrl}))}{S_{pooled}} \cdot J$$

**Time-specific pooled standard deviation:**
$$S_{pooled} = \sqrt{\frac{((n_{Exp} - 1)(sd_{t0,Exp}^2 + sd_{t1,Exp}^2) + (n_{Ctrl} - 1)(sd_{t0,Ctrl}^2 + sd_{t1,Ctrl}^2))}{2(n_{Exp} + n_{Ctrl} - 2)}}$$

**Sampling variance:**
$$var(d) = \frac{2(1 - r_{Exp})}{n_{Exp}} + \frac{2(1 - r_{Ctrl})}{n_{Ctrl}} + \frac{d^2}{2(n_{Exp} + n_{Ctrl})}$$

where $r_{Exp}$ and $r_{Ctrl}$ are the correlations between time points within each group.

### Treatment × Time lnRR: `time_lnRR()` {#time_lnrr}

**Formula:**
$$\ln RR = \ln\left(\frac{\bar{X}_{t1,Exp} / \bar{X}_{t1,Ctrl}}{\bar{X}_{t0,Exp} / \bar{X}_{t0,Ctrl}}\right)$$

**Sampling variance:**
$$var(\ln RR) = \frac{(sd_{t0,Exp}^2 \bar{X}_{t1,Exp}^2 + sd_{t1,Exp}^2 \bar{X}_{t0,Exp}^2 - 2r_{Exp} \bar{X}_{t0,Exp} \bar{X}_{t1,Exp} sd_{t0,Exp} sd_{t1,Exp})}{n_{Exp} \bar{X}_{t0,Exp}^2 \bar{X}_{t1,Exp}^2} +$$
$$\frac{(sd_{t0,Ctrl}^2 \bar{X}_{t1,Ctrl}^2 + sd_{t1,Ctrl}^2 \bar{X}_{t0,Ctrl}^2 - 2r_{Ctrl} \bar{X}_{t0,Ctrl} \bar{X}_{t1,Ctrl} sd_{t0,Ctrl} sd_{t1,Ctrl})}{n_{Ctrl} \bar{X}_{t0,Ctrl}^2 \bar{X}_{t1,Ctrl}^2}$$

### Treatment × Time lnVR: `time_lnVR()` {#time_lnvr}

**Formula:**
$$\ln VR = \ln\left(\frac{sd_{t1,Exp} / sd_{t1,Ctrl}}{sd_{t0,Exp} / sd_{t0,Ctrl}}\right)$$

**Sampling variance:**
$var(\ln VR) = \frac{(1 - r_{Exp}^2)}{n_{Exp} - 1} + \frac{(1 - r_{Ctrl}^2)}{n_{Ctrl} - 1}$

### Treatment × Time lnCVR: `time_lnCVR()` {#time_lncvr}

**Formula:**
$$\ln CVR = \ln\left(\frac{CV_{t1,Exp} / CV_{t1,Ctrl}}{CV_{t0,Exp} / CV_{t0,Ctrl}}\right)$$

**Sampling variance:**
$$var(\ln CVR) = var(\ln RR) + var(\ln VR)$$

---

## Function Parameters

All functions share common parameters:

- **`data`**: Data frame containing the variables
- **`col_names`**: Vector of two strings naming the output columns (default: `c("yi", "vi")`)
- **`append`**: Logical, whether to append results to input data (default: `TRUE`)

### Factorial Design Parameters

- **Group statistics**: `[Group]_mean`, `[Group]_sd`, `[Group]_n` for each group (Ctrl, A, B, AB)
- **`hedges_correction`**: Logical, apply small-sample bias correction (SMD only, default: `TRUE`)

### Repeated Measures Parameters

- **Time-specific statistics**: `t0_[Group]_mean`, `t0_[Group]_sd`, `t1_[Group]_mean`, `t1_[Group]_sd`
- **Sample sizes**: `[Group]_n` for each group
- **Correlations**: `[Group]_cor` - correlation between time points within each group

---

## References

- Gurevitch, J., Morrison, J. A., & Hedges, L. V. (2000). The interaction between competition and predation: a meta-analysis of field experiments. *The American Naturalist*, 155(4), 435-453.

- Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A., Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive effects of enemies and mutualists on plant performance: a meta‐analysis. *Ecology*, 88(4), 1021-1029.

- Nakagawa, S., Poulin, R., Mengersen, K., Reinhold, K., Engqvist, L., Lagisz, M., & Senior, A. M. (2015). Meta‐analysis of variation: ecological and evolutionary applications and beyond. *Methods in Ecology and Evolution*, 6(2), 143-152.

- Lajeunesse, M. J. (2011). On the meta‐analysis of response ratios for studies with correlated and multi‐group designs. *Ecology*, 92(11), 2049-2055.

*Personal communications: Shinichi Nakagawa and Daniel Noble for recent developments in factorial effect size calculations.*

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lnVR_wrappers.R
\docType{data}
\name{.lnVR_args}
\alias{.lnVR_args}
\title{Required columns for computing lnVR}
\format{
An object of class \code{list} of length 2.
}
\usage{
.lnVR_args
}
\description{
'inter' use the same arguments than 'main',
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lnRR_wrappers.R
\name{lnRR_main}
\alias{lnRR_main}
\title{Main effect: Log Response Ratio}
\usage{
lnRR_main(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  method = "nakagawa",
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{method}{Method to compute lnRR. Can be either "nakagawa" or "morris". Default is "nakagawa".}

\item{Ctrl_mean}{Mean outcome from the Control treatment}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_mean}{Mean outcome from the A treatment}

\item{A_sd}{Standard deviation from the A treatment}

\item{A_n}{Sample size from the A treatment}

\item{B_mean}{Mean outcome from the B treatment}

\item{B_sd}{Standard deviation from the B treatment}

\item{B_n}{Sample size from the B treatment}

\item{AB_mean}{Mean outcome from the interaction AxB treatment}

\item{AB_sd}{Standard deviation from the interaction AxB treatment}

\item{AB_n}{Sample size from the interaction AxB treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the main effect of Factor A across levels of Factor B, analogous
to the main effect in a factorial ANOVA.
}
\details{
See the package vignette for a detailed description of the formula.
}
\examples{
# Example data for 2x2 factorial design (Fertilization x Warming)
data <- data.frame(
  study_id = 1:2,
  control_mean = c(10, 12), control_sd = c(2.0, 2.5), control_n = c(20, 18),
  fertilization_mean = c(15, 16), fertilization_sd = c(2.2, 2.8), fertilization_n = c(20, 19),
  warming_mean = c(11, 13), warming_sd = c(2.1, 2.6), warming_n = c(21, 17),
  fert_warm_mean = c(17, 19), fert_warm_sd = c(2.4, 3.0), fert_warm_n = c(19, 20)
)

# Compute main effect of fertilization
result <- lnRR_main(
  data = data,
  Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
  A_mean = "fertilization_mean", A_sd = "fertilization_sd", A_n = "fertilization_n",
  B_mean = "warming_mean", B_sd = "warming_sd", B_n = "warming_n",
  AB_mean = "fert_warm_mean", AB_sd = "fert_warm_sd", AB_n = "fert_warm_n"
)

}
\references{
Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
effects of enemies and mutualists on plant performance: a meta‐analysis.
Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442

Lajeunesse, M. J. (2011). On the meta‐analysis of response ratios for
studies with correlated and multi‐group designs. Ecology, 92(11), 2049-2055.
https://doi.org/10.1890/11-0423.1

Macartney, E. L., Lagisz, M., & Nakagawa, S. (2022). The relative benefits
of environmental enrichment on learning and memory are greater when
stressed: A meta-analysis of interactions in rodents.
Neuroscience & Biobehavioral Reviews, 135, 104554.
https://doi.org/10.1016/j.neubiorev.2022.104554
}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/time_lnVR.R
\name{.time_interaction_lnVR}
\alias{.time_interaction_lnVR}
\title{Log of Variability Ratio: Interaction Between Experimental Treatment and Time}
\usage{
.time_interaction_lnVR(
  t0_Ctrl_sd,
  t1_Ctrl_sd,
  Ctrl_n,
  Ctrl_cor,
  t0_Exp_sd,
  t1_Exp_sd,
  Exp_n,
  Exp_cor
)
}
\arguments{
\item{t0_Ctrl_sd}{Standard deviation from the control group at time 0}

\item{t1_Ctrl_sd}{Standard deviation from the control group at time 1}

\item{Ctrl_n}{Sample size of the control group}

\item{Ctrl_cor}{Correlation between the means of the control group at t0 and t1}

\item{t0_Exp_sd}{Standard deviation from the experimental group at time 0}

\item{t1_Exp_sd}{Standard deviation from the experimental group at time 1}

\item{Exp_n}{Sample size of the experimental group}

\item{Exp_cor}{Correlation between the means of the experimental group at t0 and t1}
}
\description{
Log of Variability Ratio: Interaction Between Experimental Treatment and Time
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SMD.R
\name{.interaction_SMD}
\alias{.interaction_SMD}
\title{Interaction effect: Standardized mean difference}
\usage{
.interaction_SMD(
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n,
  hedges_correction = TRUE
)
}
\arguments{
\item{Ctrl_mean}{Mean outcome from the Control treatment}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_mean}{Mean outcome from the treatment}

\item{A_sd}{Standard deviation from the treatment}

\item{A_n}{Sample size from the treatment}

\item{B_mean}{Mean outcome from the B treatment}

\item{B_sd}{Standard deviation from the B treatment}

\item{B_n}{Sample size from the B treatment}

\item{AB_mean}{Mean outcome from the interaction AxB treatment}

\item{AB_sd}{Standard deviation from the interaction AxB treatment}

\item{AB_n}{Sample size from the interaction AxB treatment}

\item{hedges_correction}{Logical. Apply or not Hedges' correction for small-sample bias. Default is TRUE}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the interaction effect between factors A and B in factorial
data.
}
\details{
See the package vignette for a detailed description of the formula.
}
\examples{
data <- data.frame(
  study_id = 1:2,
  control_mean = c(24.8, 27.2), control_sd = c(4.1, 4.6), control_n = c(18, 16),
  salinity_mean = c(19.3, 21.7), salinity_sd = c(3.8, 4.2), salinity_n = c(17, 18),
  temperature_mean = c(28.9, 31.4), temperature_sd = c(4.7, 5.1), temperature_n = c(19, 15),
  salt_temp_mean = c(15.2, 17.8), salt_temp_sd = c(3.1, 3.5), salt_temp_n = c(16, 17)
)

result <- SMD_inter(
  data = data,
  Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
  A_mean = "salinity_mean", A_sd = "salinity_sd", A_n = "salinity_n",
  B_mean = "temperature_mean", B_sd = "temperature_sd", B_n = "temperature_n",
  AB_mean = "salt_temp_mean", AB_sd = "salt_temp_sd", AB_n = "salt_temp_n"
)

}
\references{
Gurevitch, J., Morrison, J. A., & Hedges, L. V. (2000). The interaction
between competition and predation: a meta-analysis of field experiments.
The American Naturalist, 155(4), 435-453.

Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
effects of enemies and mutualists on plant performance: a meta‐analysis.
Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SMD_wrappers.R
\name{SMD_main}
\alias{SMD_main}
\title{Main effect: Standardized Mean Difference}
\usage{
SMD_main(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  hedges_correction = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{hedges_correction}{Boolean. If TRUE correct for small-sample bias. Default is TRUE.}

\item{Ctrl_mean}{Mean outcome from the Control treatment}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_mean}{Mean outcome from the A treatment}

\item{A_sd}{Standard deviation from the A treatment}

\item{A_n}{Sample size from the A treatment}

\item{B_mean}{Mean outcome from the B treatment}

\item{B_sd}{Standard deviation from the B treatment}

\item{B_n}{Sample size from the B treatment}

\item{AB_mean}{Mean outcome from the interaction AxB treatment}

\item{AB_sd}{Standard deviation from the interaction AxB treatment}

\item{AB_n}{Sample size from the interaction AxB treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the main effect of Factor A across levels of Factor B, analogous
to the main effect in a factorial ANOVA.
}
\details{
See the package vignette for a detailed description of the formula.
}
\examples{
# Main effect of Mycorrhiza in 2x2 factorial design (AMF x Phosphorus)
data <- data.frame(
  study_id = 1:2,
  control_mean = c(12.4, 15.1), control_sd = c(2.8, 3.2), control_n = c(16, 14),
  mycorrhizae_mean = c(18.7, 21.3), mycorrhizae_sd = c(3.4, 3.9), mycorrhizae_n = c(15, 16),
  phosphorus_mean = c(14.9, 17.8), phosphorus_sd = c(3.1, 3.6), phosphorus_n = c(17, 13),
  myco_phos_mean = c(22.1, 25.4), myco_phos_sd = c(4.2, 4.8), myco_phos_n = c(14, 15)
)

result <- SMD_main(
  data = data,
  Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
  A_mean = "mycorrhizae_mean", A_sd = "mycorrhizae_sd", A_n = "mycorrhizae_n",
  B_mean = "phosphorus_mean", B_sd = "phosphorus_sd", B_n = "phosphorus_n",
  AB_mean = "myco_phos_mean", AB_sd = "myco_phos_sd", AB_n = "myco_phos_n"
)

}
\references{
Gurevitch, J., Morrison, J. A., & Hedges, L. V. (2000). The interaction
between competition and predation: a meta-analysis of field experiments.
The American Naturalist, 155(4), 435-453.

Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
effects of enemies and mutualists on plant performance: a meta‐analysis.
Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lnVR_wrappers.R
\name{lnVR_inter}
\alias{lnVR_inter}
\title{Interaction effect: Log Variability Ratio}
\usage{
lnVR_inter(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n,
  B_sd,
  B_n,
  AB_sd,
  AB_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_sd}{Standard deviation from the A treatment}

\item{A_n}{Sample size from the A treatment}

\item{B_sd}{Standard deviation from the B treatment}

\item{B_n}{Sample size from the B treatment}

\item{AB_sd}{Standard deviation from the interaction AxB treatment}

\item{AB_n}{Sample size from the interaction AxB treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the interaction of Factors A and B measured as
the log of the variability ratio.
}
\details{
See the package vignette for a detailed description of the formula.
}
\examples{
# Example for interaction effect in 2x2 factorial focusing on variability (Drought x Temperature)
data <- data.frame(
  study_id = 1:2,
  control_sd = c(1.8, 2.1), control_n = c(22, 19),
  drought_sd = c(2.6, 2.9), drought_n = c(20, 21),
  temperature_sd = c(2.0, 2.3), temperature_n = c(21, 18),
  drought_temp_sd = c(3.2, 3.6), drought_temp_n = c(19, 20)
)

result <- lnVR_inter(
  data = data,
  Ctrl_sd = "control_sd", Ctrl_n = "control_n",
  A_sd = "drought_sd", A_n = "drought_n", 
  B_sd = "temperature_sd", B_n = "temperature_n",
  AB_sd = "drought_temp_sd", AB_n = "drought_temp_n"
)

}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SMD_wrappers.R
\name{SMD_ind}
\alias{SMD_ind}
\title{Simple effect: Standardized Mean Difference}
\usage{
SMD_ind(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  hedges_correction = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{hedges_correction}{Boolean. If TRUE correct for small-sample bias. Default is TRUE.}

\item{Ctrl_mean}{Mean outcome from the Control treatment}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_mean}{Mean outcome from the experimental treatment}

\item{A_sd}{Standard deviation from the experimental treatment}

\item{A_n}{Sample size from the experimental treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the individual or simple effect of Factor A over the Control.
}
\details{
It is the classic Standardized Mean Difference (SMD), which can also be computed
with metafor's \code{escalc()} function using \code{measure = "SMD"}.

See the package vignette for a detailed description of the formula.
}
\examples{
data <- data.frame(
  study_id = 1:3,
  control_mean = c(45.2, 52.8, 38.9),
  control_sd = c(8.1, 11.2, 7.3),
  control_n = c(18, 23, 16),
  pollinator_exclusion_mean = c(28.7, 35.4, 22.1),
  pollinator_exclusion_sd = c(6.8, 9.1, 5.9),
  pollinator_exclusion_n = c(20, 22, 18)
)

# With Hedges' correction (default)
result <- SMD_ind(
  data = data,
  Ctrl_mean = "control_mean",
  Ctrl_sd = "control_sd",
  Ctrl_n = "control_n",
  A_mean = "pollinator_exclusion_mean",
  A_sd = "pollinator_exclusion_sd",
  A_n = "pollinator_exclusion_n",
  hedges_correction = TRUE
)

# Without Hedges' correction
result_no_hedges <- SMD_ind(
  data = data,
  Ctrl_mean = "control_mean",
  Ctrl_sd = "control_sd",
  Ctrl_n = "control_n",
  A_mean = "pollinator_exclusion_mean",
  A_sd = "pollinator_exclusion_sd",
  A_n = "pollinator_exclusion_n",
  hedges_correction = FALSE
)

}
\references{
Gurevitch, J., Morrison, J. A., & Hedges, L. V. (2000). The interaction
between competition and predation: a meta-analysis of field experiments.
The American Naturalist, 155(4), 435-453.

Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
effects of enemies and mutualists on plant performance: a meta‐analysis.
Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{testing_data}
\alias{testing_data}
\title{Fake factorial dataset with pre-computed effect sizes}
\format{
A data frame with 10 rows and 24 columns:
\describe{
\item{C_n, C_mean, C_sd}{Sample size, mean, and SD of the Control group}
\item{A_n, A_mean, A_sd}{Sample size, mean, and SD of the A treatment}
\item{B_n, B_mean, B_sd}{Sample size, mean, and SD of the B treatment}
\item{AB_n, AB_mean, AB_sd}{Sample size, mean, and SD of the AB (A and B combined) treatment}
\item{A_main_lnRR, A_main_lnRRv}{Log response ratio for main effect A and its variance}
\item{B_main_lnRR, B_main_lnRRv}{Log response ratio for main effect B and its variance}
\item{AB_main_lnRR, AB_main_lnRRv}{Log response ratio for the AB interaction and its variance}
\item{A_main_SMD, A_main_SMDv}{Standardized mean difference (Hedges' g) for main effect A and its variance}
\item{B_main_SMD, B_main_SMDv}{Standardized mean difference for main effect B and its variance}
\item{AB_main_SMD, AB_main_SMDv}{Standardized mean difference for the AB interaction and its variance}
}
}
\source{
Fake.
}
\usage{
testing_data
}
\description{
This dataset contains fake data from 2×2 factorial experiments with four treatment groups:
Control (C), Factor A, Factor B, and the combination AB (A and B together). It includes means,
standard deviations, and sample sizes for each group, as well as pre-computed effect sizes:
log response ratio (lnRR), standardized mean difference (SMD), and their variances, for the main
effects of A, B, and the interaction (AB).
}
\details{
This dataset is intended for validating functions that compute or test effect sizes
in meta-analysis workflows involving factorial designs.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/time_SMD.R
\name{time_SMD}
\alias{time_SMD}
\title{Standardized Mean Difference: Interaction Between Treatment and Time}
\usage{
time_SMD(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  hedges_correction = TRUE,
  t0_Ctrl_mean,
  t0_Ctrl_sd,
  t1_Ctrl_mean,
  t1_Ctrl_sd,
  Ctrl_n,
  Ctrl_cor,
  t0_Exp_mean,
  t0_Exp_sd,
  t1_Exp_mean,
  t1_Exp_sd,
  Exp_n,
  Exp_cor
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{hedges_correction}{Logical. Apply or not Hedges' correction for small-sample bias. Default is TRUE.}

\item{t0_Ctrl_mean}{Sample mean from the control group at time 0}

\item{t0_Ctrl_sd}{Standard deviation from the control group at time 0}

\item{t1_Ctrl_mean}{Sample mean from the control group at time 1}

\item{t1_Ctrl_sd}{Standard deviation from the control group at time 1}

\item{Ctrl_n}{Sample size of the control group}

\item{Ctrl_cor}{Number or numeric vector. Correlation between the means of the control group at t0 and t1}

\item{t0_Exp_mean}{Sample mean from the experimental group at time 0}

\item{t0_Exp_sd}{Standard deviation from the experimental group at time 0}

\item{t1_Exp_mean}{Sample mean from the experimental group at time 1}

\item{t1_Exp_sd}{Standard deviation from the experimental group at time 1}

\item{Exp_n}{Sample size of the experimental group}

\item{Exp_cor}{Number or numeric vector. Correlation between the means of the experimental group at t0 and t1}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Standardized Mean Difference: Interaction Between Treatment and Time
}
\examples{
# Pre-post design for standardized mean difference with time interaction (Conservation experiment)
data <- data.frame(
  study_id = 1:2,
  pre_control_mean = c(18.3, 21.7), pre_control_sd = c(4.1, 4.8),
  post_control_mean = c(18.8, 22.1), post_control_sd = c(4.2, 4.9),
  control_n = c(16, 14),
  pre_conservation_mean = c(18.1, 21.4), pre_conservation_sd = c(4.0, 4.7),
  post_conservation_mean = c(26.7, 31.2), post_conservation_sd = c(5.8, 6.4),
  conservation_n = c(15, 16)
)

result <- time_SMD(
  data = data,
  t0_Ctrl_mean = "pre_control_mean", t0_Ctrl_sd = "pre_control_sd",
  t1_Ctrl_mean = "post_control_mean", t1_Ctrl_sd = "post_control_sd",
  Ctrl_n = "control_n", Ctrl_cor = 0.9,
  t0_Exp_mean = "pre_conservation_mean", t0_Exp_sd = "pre_conservation_sd",
  t1_Exp_mean = "post_conservation_mean", t1_Exp_sd = "post_conservation_sd",
  Exp_n = "conservation_n", Exp_cor = 0.7,
  hedges_correction = TRUE
)

# Without Hedges' correction
result_no_hedges <- time_SMD(
  data = data,
  t0_Ctrl_mean = "pre_control_mean", t0_Ctrl_sd = "pre_control_sd",
  t1_Ctrl_mean = "post_control_mean", t1_Ctrl_sd = "post_control_sd",
  Ctrl_n = "control_n", Ctrl_cor = 0.9,
  t0_Exp_mean = "pre_conservation_mean", t0_Exp_sd = "pre_conservation_sd",
  t1_Exp_mean = "post_conservation_mean", t1_Exp_sd = "post_conservation_sd",
  Exp_n = "conservation_n", Exp_cor = 0.7,
  hedges_correction = FALSE
)

}
\references{
Shinichi Nakagawa and Daniel Noble, personal communication.
}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lnRR_wrappers.R
\name{lnRR_inter}
\alias{lnRR_inter}
\title{Interaction effect: Log Response Ratio}
\usage{
lnRR_inter(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{Ctrl_mean}{Mean outcome from the Control treatment}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_mean}{Mean outcome from the treatment}

\item{A_sd}{Standard deviation from the treatment}

\item{A_n}{Sample size from the treatment}

\item{B_mean}{Mean outcome from the B treatment}

\item{B_sd}{Standard deviation from the B treatment}

\item{B_n}{Sample size from the B treatment}

\item{AB_mean}{Mean outcome from the interaction AxB treatment}

\item{AB_sd}{Standard deviation from the interaction AxB treatment}

\item{AB_n}{Sample size from the interaction AxB treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the interaction effect between factors A and B in factorial
data.
}
\details{
See the package vignette for a detailed description of the formula.
}
\examples{
data <- data.frame(
  study_id = 1:2,
  control_mean = c(25, 28), control_sd = c(3.2, 3.8), control_n = c(15, 17),
  predation_mean = c(18, 20), predation_sd = c(2.9, 3.1), predation_n = c(16, 18),
  competition_mean = c(22, 24), competition_sd = c(3.0, 3.5), competition_n = c(14, 16),
  pred_comp_mean = c(12, 15), pred_comp_sd = c(2.1, 2.6), pred_comp_n = c(15, 17)
)

# Compute interaction effect between predation and competition
result <- lnRR_inter(
  data = data,
  Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
  A_mean = "predation_mean", A_sd = "predation_sd", A_n = "predation_n",
  B_mean = "competition_mean", B_sd = "competition_sd", B_n = "competition_n",
  AB_mean = "pred_comp_mean", AB_sd = "pred_comp_sd", AB_n = "pred_comp_n"
)

}
\references{
Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
effects of enemies and mutualists on plant performance: a meta‐analysis.
Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lnRR_wrappers.R
\name{lnRR_ind}
\alias{lnRR_ind}
\title{Simple effect: Log Response Ratio}
\usage{
lnRR_ind(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{Ctrl_mean}{Mean outcome from the Control treatment}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_mean}{Mean outcome from the experimental treatment}

\item{A_sd}{Standard deviation from the experimental treatment}

\item{A_n}{Sample size from the experimental treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the individual or simple effect of Factor A over the Control.
}
\details{
It is the classic Log Response Ratio (lnRR), which can also be computed
with metafor's \code{escalc()} function using \code{measure = "ROM"}.

See the package vignette for a detailed description of the formula.
}
\examples{
data <- data.frame(
  study_id = 1:3,
  control_mean = c(10, 15, 12),
  control_sd = c(2.1, 3.2, 2.8),
  control_n = c(20, 25, 18),
  drought_mean = c(12, 18, 14),
  drought_sd = c(2.3, 3.5, 3.1),
  drought_n = c(22, 24, 20)
)

# Compute individual effect of drought vs control
result <- lnRR_ind(
  data = data,
  Ctrl_mean = "control_mean",
  Ctrl_sd = "control_sd", 
  Ctrl_n = "control_n",
  A_mean = "drought_mean",
  A_sd = "drought_sd",
  A_n = "drought_n"
)

}
\references{
Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
effects of enemies and mutualists on plant performance: a meta‐analysis.
Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442

Lajeunesse, M. J. (2011). On the meta‐analysis of response ratios for
studies with correlated and multi‐group designs. Ecology, 92(11), 2049-2055.
https://doi.org/10.1890/11-0423.1
}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lnCVR_wrappers.R
\name{lnCVR_main}
\alias{lnCVR_main}
\title{Main Effect: Log Coefficient Of Variation Ration}
\usage{
lnCVR_main(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{Ctrl_mean}{Mean outcome from the Control treatment}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_mean}{Mean outcome from the treatment}

\item{A_sd}{Standard deviation from the treatment}

\item{A_n}{Sample size from the treatment}

\item{B_mean}{Mean outcome from the B treatment}

\item{B_sd}{Standard deviation from the B treatment}

\item{B_n}{Sample size from the B treatment}

\item{AB_mean}{Mean outcome from the interaction AxB treatment}

\item{AB_sd}{Standard deviation from the interaction AxB treatment}

\item{AB_n}{Sample size from the interaction AxB treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the main effect of Factor A across levels of Factor B
in factorial experiments on the coefficient of variation.
}
\details{
See the package vignette for a detailed description of the formula.
}
\examples{
data <- data.frame(
  study_id = 1:2,
  control_mean = c(14.2, 16.8), control_sd = c(2.8, 3.1), control_n = c(16, 14),
  irrigation_mean = c(19.5, 22.1), irrigation_sd = c(5.2, 5.8), irrigation_n = c(15, 16),
  co2_mean = c(16.8, 19.4), co2_sd = c(3.1, 3.6), co2_n = c(17, 13),
  irrigation_co2_mean = c(24.3, 27.9), irrigation_co2_sd = c(6.8, 7.4), irrigation_co2_n = c(14, 15)
)

result <- lnCVR_main(
  data = data,
  Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
  A_mean = "irrigation_mean", A_sd = "irrigation_sd", A_n = "irrigation_n",
  B_mean = "co2_mean", B_sd = "co2_sd", B_n = "co2_n", 
  AB_mean = "irrigation_co2_mean", AB_sd = "irrigation_co2_sd", AB_n = "irrigation_co2_n"
)

}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lnVR_wrappers.R
\name{lnVR_main}
\alias{lnVR_main}
\title{Main Effect: Log of the Variability Ratio}
\usage{
lnVR_main(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n,
  B_sd,
  B_n,
  AB_sd,
  AB_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_sd}{Standard deviation from the A treatment}

\item{A_n}{Sample size from the A treatment}

\item{B_sd}{Standard deviation from the B treatment}

\item{B_n}{Sample size from the B treatment}

\item{AB_sd}{Standard deviation from the interaction AxB treatment}

\item{AB_n}{Sample size from the interaction AxB treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the overral log of the variability ratio for Factor A
across levels of Factor B.
}
\details{
See the package vignette for a detailed description of the formula.
}
\examples{
# Example for main effect in 2x2 factorial focusing on variability (Fire x Grazing)
data <- data.frame(
  study_id = 1:2,
  control_sd = c(2.0, 2.3), control_n = c(20, 18),
  fire_sd = c(2.8, 3.1), fire_n = c(19, 20),
  grazing_sd = c(2.2, 2.5), grazing_n = c(21, 17),
  fire_grazing_sd = c(3.5, 3.8), fire_grazing_n = c(18, 19)
)

result <- lnVR_main(
  data = data,
  Ctrl_sd = "control_sd", Ctrl_n = "control_n",
  A_sd = "fire_sd", A_n = "fire_n",
  B_sd = "grazing_sd", B_n = "grazing_n",
  AB_sd = "fire_grazing_sd", AB_n = "fire_grazing_n"
)

}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SMD.R
\name{.pooled_sd}
\alias{.pooled_sd}
\title{Pooled Standard Deviation for SMD in factorial experiments}
\usage{
.pooled_sd(
  Ctrl_sd,
  Ctrl_n,
  A_sd,
  A_n,
  B_sd = NULL,
  B_n = NULL,
  AB_n = NULL,
  AB_sd = NULL
)
}
\arguments{
\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_sd}{Standard deviation from the A treatment}

\item{A_n}{Sample size from the A treatment}

\item{B_sd}{Standard deviation from the B treatment}

\item{B_n}{Sample size from the B treatment}

\item{AB_n}{Sample size from the interaction AxB treatment}

\item{AB_sd}{Standard deviation from the interaction AxB treatment}
}
\description{
Compute the pooled standard deviation for SMD. It computes the pooled SD
for 2 or 4 groups depending on the arguments passed.
Simple SMD has only 2 groups, while main and interactions had 4 groups.
}
\references{
Morris, W. F., Hufbauer, R. A., Agrawal, A. A., Bever, J. D., Borowicz, V. A.,
Gilbert, G. S., ... & Vázquez, D. P. (2007). Direct and interactive
effects of enemies and mutualists on plant performance: a meta‐analysis.
Ecology, 88(4), 1021-1029. https://doi.org/10.1890/06-0442
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lnCVR_wrappers.R
\name{lnCVR_inter}
\alias{lnCVR_inter}
\title{Interaction Effect: Log Coefficient of Variation Ratio}
\usage{
lnCVR_inter(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n,
  B_mean,
  B_sd,
  B_n,
  AB_mean,
  AB_sd,
  AB_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{Ctrl_mean}{Mean outcome from the Control treatment}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_mean}{Mean outcome from the treatment}

\item{A_sd}{Standard deviation from the treatment}

\item{A_n}{Sample size from the treatment}

\item{B_mean}{Mean outcome from the B treatment}

\item{B_sd}{Standard deviation from the B treatment}

\item{B_n}{Sample size from the B treatment}

\item{AB_mean}{Mean outcome from the interaction AxB treatment}

\item{AB_sd}{Standard deviation from the interaction AxB treatment}

\item{AB_n}{Sample size from the interaction AxB treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the interaction effect between Factors A and B
in factorial experiments on the coefficient of variation ratio.
}
\details{
See the package vignette for a detailed description of the formula.
}
\examples{
# Interaction effect logCVR (Light x Nutrients)
data <- data.frame(
  study_id = 1:2,
  control_mean = c(7.3, 8.9),
  control_sd = c(1.4, 1.7),
  control_n = c(20, 18),
  light_mean = c(12.8, 14.2),
  light_sd = c(3.1, 3.5),
  light_n = c(19, 20),
  nutrients_mean = c(9.6, 11.1),
  nutrients_sd = c(1.9, 2.2),
  nutrients_n = c(21, 17),
  light_nutrients_mean = c(18.4, 20.7),
  light_nutrients_sd = c(4.8, 5.3),
  light_nutrients_n = c(18, 19)
)

result <- lnCVR_inter(
  data = data,
  Ctrl_mean = "control_mean",
  Ctrl_sd = "control_sd",
  Ctrl_n = "control_n",
  A_mean = "light_mean",
  A_sd = "light_sd",
  A_n = "light_n",
  B_mean = "nutrients_mean",
  B_sd = "nutrients_sd",
  B_n = "nutrients_n",
  AB_mean = "light_nutrients_mean",
  AB_sd = "light_nutrients_sd",
  AB_n = "light_nutrients_n"
)

}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{.compute_and_format}
\alias{.compute_and_format}
\title{Compute And Format}
\usage{
.compute_and_format(effsize_func, effsize_args, data, col_names, append)
}
\arguments{
\item{effsize_func}{Function used to compute the effect size}

\item{effsize_args}{Arguments needed for that function}

\item{data}{Data frame to use}

\item{col_names}{Vector of length 2 with the name of the columns}

\item{append}{Logical. If append or not the effect sizes to the data}
}
\description{
Compute effect size and do some pre-return tasks
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/time_SMD.R
\name{.time_pooled_sd}
\alias{.time_pooled_sd}
\title{Pooled Standard Deviation for SMD in non-independent factorial}
\usage{
.time_pooled_sd(t0_Ctrl_sd, t1_Ctrl_sd, Ctrl_n, t0_Exp_sd, t1_Exp_sd, Exp_n)
}
\arguments{
\item{t0_Ctrl_sd}{Standard deviation from Control treatment at time 0}

\item{t1_Ctrl_sd}{Standard deviation from Control treatment at time 1}

\item{Ctrl_n}{Sample size from Control treatment}

\item{t0_Exp_sd}{Standard deviation from Experimental treatment at time 0}

\item{t1_Exp_sd}{Standard deviation from Experimental treatment at time 1}

\item{Exp_n}{Sample size from Experimental treatment}
}
\description{
Pooled Standard Deviation for SMD in non-independent factorial
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lnCVR_wrappers.R
\name{lnCVR_ind}
\alias{lnCVR_ind}
\title{Individual Effect: Log Coefficient Of Variation Ratio}
\usage{
lnCVR_ind(
  data,
  col_names = c("yi", "vi"),
  append = TRUE,
  Ctrl_mean,
  Ctrl_sd,
  Ctrl_n,
  A_mean,
  A_sd,
  A_n
)
}
\arguments{
\item{data}{Data frame containing the variables used.}

\item{col_names}{Vector of two strings to name the output columns for the effect size and its sampling variance. Default is 'yi' and 'vi'.}

\item{append}{Logical. Append the results to \code{data}. Default is TRUE}

\item{Ctrl_mean}{Mean outcome from the Control treatment}

\item{Ctrl_sd}{Standard deviation from the control treatment}

\item{Ctrl_n}{Sample size from the control treatment}

\item{A_mean}{Mean outcome from the treatment}

\item{A_sd}{Standard deviation from the treatment}

\item{A_n}{Sample size from the treatment}
}
\value{
A data frame containing the effect sizes and their sampling variance.
By default, the columns are named \code{yi} (effect size) and \code{vi} (sampling variance).
If \code{append = TRUE}, the results are appended to the input \code{data}; otherwise, only the computed effect size columns are returned.
}
\description{
Computes the Log of the Coefficient of Variation Ratio between Factor A
and the Control treatment.
}
\details{
See the package vignette for a detailed description of the formula.
}
\examples{
data <- data.frame(
  study_id = 1:3,
  control_mean = c(8.5, 12.3, 6.8),
  control_sd = c(1.8, 2.9, 1.4),
  control_n = c(18, 24, 16),
  nutrient_mean = c(11.2, 16.7, 9.3),
  nutrient_sd = c(3.1, 4.8, 2.7),
  nutrient_n = c(19, 22, 17)
)

result <- lnCVR_ind(
  data = data,
  Ctrl_mean = "control_mean", Ctrl_sd = "control_sd", Ctrl_n = "control_n",
  A_mean = "nutrient_mean", A_sd = "nutrient_sd", A_n = "nutrient_n"
)

}
\references{
Nakagawa, S., Poulin, R., Mengersen, K., Reinhold, K., Engqvist,
L., Lagisz, M., & Senior, A. M. (2015). Meta‐analysis of variation:
ecological and evolutionary applications and beyond. Methods in
Ecology and Evolution, 6(2), 143-152.
}
\author{
Facundo Decunta - fdecunta@agro.uba.ar
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SMD.R
\name{.j_correction}
\alias{.j_correction}
\title{Correction for small-sample bias}
\usage{
.j_correction(m)
}
\arguments{
\item{m}{Degrees of freedom. They change between individual and main effects.}
}
\description{
Small-sample bias correction for Hedges' g
}
\keyword{internal}
